"""
Sistema de respuestas estandarizadas para tai-api.

Este módulo proporciona clases para manejar respuestas exitosas y de error
de manera consistente en toda la API.
"""
# Importar excepciones y manejadores
from .exceptions import (
    ErrorCode,
    APIException,
    ValidationException,
    RecordNotFoundException,
    DatabaseException,
    DuplicateRecordException,
    ForeignKeyViolationException,
    BusinessRuleViolationException,
    InvalidCredentialsException,
    InvalidTokenException,
    TokenExpiredException,
    SessionInvalidatedException,
    SessionExpiredException,
    ConcurrentSessionDetectedException
)
from .responses import (
    APIResponse,
    APIError,
    PaginationMeta,
    ResponseStatus,
    PaginatedResponse
)

from .handlers import setup_exception_handlers

__all__ = [
    # Clases de respuesta
    "ResponseStatus",
    "APIError",
    "PaginationMeta",
    "APIResponse",
    "PaginatedResponse",
    
    # Excepciones
    "APIException",
    "ErrorCode", 
    "ValidationException", 
    "RecordNotFoundException",
    "DatabaseException",
    "DuplicateRecordException",
    "ForeignKeyViolationException",
    "BusinessRuleViolationException",
    "InvalidCredentialsException",
    "InvalidTokenException",
    "TokenExpiredException", 
    "SessionInvalidatedException",
    "SessionExpiredException",
    "ConcurrentSessionDetectedException",
    
    # Manejadores
    "setup_exception_handlers",
]



"""
chat-demo - Authentication Dependencies
Generado automáticamente por tai-api set-auth

Este módulo contiene las dependencias de FastAPI para autenticación,
incluyendo get_current_user y validación de sesiones.
"""

from fastapi import Depends
from fastapi.security import OAuth2PasswordBearer
from typing import Annotated

from .jwt import JWTHandler, TokenPayload
from ..database import ChatbotAsyncDBAPI, UsuarioRead
from ..resources import (
    SessionInvalidatedException,
    RecordNotFoundException
)

# Configurar OAuth2PasswordBearer
oauth2_scheme = OAuth2PasswordBearer(tokenUrl="auth/docslogin")

async def get_current_user(
    token: Annotated[str, Depends(oauth2_scheme)],
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> UsuarioRead:
    """
    Dependencia de FastAPI para obtener el usuario actual autenticado.
    
    Esta función:
    1. Recibe el token automáticamente via OAuth2PasswordBearer
    2. Valida y decodifica el token JWT
    3. Verifica que el usuario existe en la base de datos
    4. Valida que el session_id del token coincida con el de la BD    
    Args:
        token: Token JWT extraído automáticamente del header Authorization
        
    Returns:
        UsuarioRead: Usuario autenticado
        
    Raises:
        InvalidTokenException: Si el token es inválido
        SessionInvalidatedException: Si la sesión fue invalidada
        RecordNotFoundException: Si el usuario no existe
    """
    
    # 1. Decodificar y validar token JWT
    payload: TokenPayload = JWTHandler.decode_token(token)
    
    # 2. Buscar usuario en la base de datos
    user = await api.usuario.find(
        username=payload.username
    )
    
    if not user:
        raise RecordNotFoundException("Usuario no encontrado")
    
    # 3. Validar session_id si está configurado
    
    if user.session_id != payload.session_id:
        raise SessionInvalidatedException()
    
    return user

# Alias para uso más conveniente
CurrentUser = UsuarioRead


"""
Custom ER Diagram generator for tai-chat project.
"""
import os
from tai_sql.generators import ERDiagramGenerator


class CustomERDiagramGenerator(ERDiagramGenerator):
    """
    Custom ER Diagram generator that extends the base ERDiagramGenerator.
    """
    def generate(self) -> str:
        """
        Genera el diagrama ER.
        
        Returns:
            str: Ruta del archivo generado
        """

        # Verificar dependencias
        self.check_dependencies()
        
        # Analizar modelos y crear entidades
        self.load_entities()
        
        # Detectar relaciones
        if self.include_relationships:
            self.load_relationships()
        
        # Crear diagrama Graphviz
        dot = self.create_diagram()
        
        # Renderizar
        output_path = os.path.join(self.config.output_dir, f'.diagram')
        dot.render(output_path, format=self.format, cleanup=True)
        
        final_path = f"{output_path}.{self.format}"
        
        return final_path


from langchain_core.tools import tool
from typing import Optional, List, Literal

from .schemas import *

# Añade aquí las herramientas que quieras exponer
# Por ejemplo:

@tool(args_schema=SumaParams)
async def suma(a: float, b: float) -> float:
    """
    Sumar dos números
    """
    return a + b

# Considera utilizar tai-sql para crear herramientas que interactúen con tu base de datos


import click
from .commands import (
    init,
    pushdb,
    add_user
)

@click.group()
def cli():
    """CLI para tai-chat: descubre tus datos con LLMs."""
    pass

cli.add_command(init)
cli.add_command(pushdb)
cli.add_command(add_user)
# cli.add_command(deploy)

if __name__ == '__main__':
    cli()


# Este archivo ha sido generado automáticamente por tai-sql
# No modifiques este archivo directamente

from __future__ import annotations
from typing import List, Optional
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column, relationship
from datetime import datetime, date, time
from sqlalchemy import BigInteger, ForeignKeyConstraint
import os
from cryptography.fernet import Fernet
from sqlalchemy.ext.hybrid import hybrid_property
import base64


# Configuración de encriptación
_secret_key = os.getenv('SECRET_KEY')
if not _secret_key:
    raise ValueError(
        f"Variable de entorno 'SECRET_KEY' no encontrada para encriptación. "
        f"Por favor, configure la variable de entorno 'SECRET_KEY' en su sistema "
        f"con una clave secreta segura antes de ejecutar la aplicación. "
        f"Ejemplo: export SECRET_KEY='su-clave-secreta-de-32-caracteres-aqui'"
    )

# Generar clave Fernet desde la clave secreta
_fernet_key = base64.urlsafe_b64encode(_secret_key.encode()[:32].ljust(32, b'\0'))
_cipher = Fernet(_fernet_key)

def encrypt_value(value: str) -> str:
    """Encripta un valor string"""
    if value is None:
        return None
    return _cipher.encrypt(value.encode()).decode()

def decrypt_value(value: str) -> str:
    """Desencripta un valor string"""
    if value is None:
        return None
    return _cipher.decrypt(value.encode()).decode()


class Base(DeclarativeBase):
    pass

class Usuario(Base):
    __tablename__ = "usuario"

    def __init__(self, **kwargs):
        # Manejar columnas encriptadas especialmente
        if 'password' in kwargs:
            password_value = kwargs.pop('password')
            self.password = password_value

        super().__init__(**kwargs)

    username: Mapped[str] = mapped_column(primary_key=True)
    _password: Mapped[str] = mapped_column(name="password")
    
    @hybrid_property
    def password(self) -> str:
        """Propiedad encriptada para password"""
        if self._password is None:
            return None
        return decrypt_value(self._password)
    
    @password.setter
    def password(self, value: str):
        """Setter encriptado para password"""
        if value is None:
            self._password = None
        else:
            self._password = encrypt_value(str(value))
    
    email: Mapped[Optional[str]]
    avatar: Mapped[Optional[str]]
    session_id: Mapped[Optional[str]]
    created_at: Mapped[datetime] = mapped_column(default=datetime.now)
    updated_at: Mapped[datetime] = mapped_column(default=datetime.now)
    is_active: Mapped[bool] = mapped_column(default=True)
    
    chats: Mapped[List[Chat]] = relationship(back_populates="usuario")

    __table_args__ = {'schema': 'chatbot'}

class Chat(Base):
    __tablename__ = "chat"

    id: Mapped[int] = mapped_column(BigInteger, primary_key=True, autoincrement=True)
    title: Mapped[str]
    username: Mapped[str]
    created_at: Mapped[datetime] = mapped_column(default=datetime.now)
    updated_at: Mapped[datetime] = mapped_column(default=datetime.now)
    is_active: Mapped[bool] = mapped_column(default=True)
    
    messages: Mapped[List[Mensaje]] = relationship(back_populates="chat")
    usuario: Mapped[Usuario] = relationship(back_populates="chats")

    __table_args__ = (
        # Clave foránea simple: fk_chat_usuario
        ForeignKeyConstraint(
            # Columnas locales
            ['username'],
            # Columnas objetivo
            ['chatbot.usuario.username'],
            ondelete='CASCADE',
            onupdate='CASCADE'
        ),
        # Esquema de la tabla
        {'schema': 'chatbot'}
    )

class Mensaje(Base):
    __tablename__ = "mensaje"

    id: Mapped[int] = mapped_column(BigInteger, primary_key=True, autoincrement=True)
    content: Mapped[str]
    role: Mapped[str]
    timestamp: Mapped[datetime] = mapped_column(default=datetime.now)
    chat_id: Mapped[int] = mapped_column(BigInteger)
    
    token_usage: Mapped[List[TokenUsage]] = relationship(back_populates="message")
    chat: Mapped[Chat] = relationship(back_populates="messages")

    __table_args__ = (
        # Clave foránea simple: fk_mensaje_chat
        ForeignKeyConstraint(
            # Columnas locales
            ['chat_id'],
            # Columnas objetivo
            ['chatbot.chat.id'],
            ondelete='CASCADE',
            onupdate='CASCADE'
        ),
        # Esquema de la tabla
        {'schema': 'chatbot'}
    )

class TokenUsage(Base):
    __tablename__ = "token_usage"

    id: Mapped[int] = mapped_column(BigInteger, primary_key=True, autoincrement=True)
    prompt_tokens: Mapped[int] = mapped_column(default=0)
    completion_tokens: Mapped[int] = mapped_column(default=0)
    total_tokens: Mapped[int] = mapped_column(default=0)
    model_name: Mapped[str]
    provider: Mapped[str]
    cost_usd: Mapped[Optional[float]]
    timestamp: Mapped[datetime] = mapped_column(default=datetime.now)
    message_id: Mapped[int] = mapped_column(BigInteger)
    
    message: Mapped[Mensaje] = relationship(back_populates="token_usage")

    __table_args__ = (
        # Clave foránea simple: fk_token_usage_mensaje
        ForeignKeyConstraint(
            # Columnas locales
            ['message_id'],
            # Columnas objetivo
            ['chatbot.mensaje.id'],
            ondelete='CASCADE',
            onupdate='CASCADE'
        ),
        # Esquema de la tabla
        {'schema': 'chatbot'}
    )

class UserStats(Base):
    __tablename__ = "user_stats"

    is_view = True
    username: Mapped[str]
    email: Mapped[str]
    total_chats: Mapped[int]
    active_chats: Mapped[int]
    total_messages: Mapped[int]
    created_at: Mapped[datetime]
    last_activity: Mapped[Optional[datetime]]

    __table_args__ = {'schema': 'chatbot'}
    __mapper_args__ = {'primary_key': [
        'username',
        'email',
        'total_chats',
        'active_chats',
        'total_messages',
        'created_at',
        'last_activity',
    ]}

class TokenConsumptionStats(Base):
    __tablename__ = "token_consumption_stats"

    is_view = True
    username: Mapped[str]
    date: Mapped[datetime]
    total_prompt_tokens: Mapped[int]
    total_completion_tokens: Mapped[int]
    total_tokens: Mapped[int]
    total_cost_usd: Mapped[Optional[float]]
    chat_count: Mapped[int]
    most_used_model: Mapped[str]
    most_used_provider: Mapped[str]

    __table_args__ = {'schema': 'chatbot'}
    __mapper_args__ = {'primary_key': [
        'username',
        'date',
        'total_prompt_tokens',
        'total_completion_tokens',
        'total_tokens',
        'total_cost_usd',
        'chat_count',
        'most_used_model',
        'most_used_provider',
    ]}

class ChatActivity(Base):
    __tablename__ = "chat_activity"

    is_view = True
    chat_id: Mapped[str]
    chat_title: Mapped[str]
    username: Mapped[str]
    message_count: Mapped[int]
    last_message_timestamp: Mapped[datetime]
    total_tokens_consumed: Mapped[int]
    is_active: Mapped[bool]

    __table_args__ = {'schema': 'chatbot'}
    __mapper_args__ = {'primary_key': [
        'chat_id',
        'chat_title',
        'username',
        'message_count',
        'last_message_timestamp',
        'total_tokens_consumed',
        'is_active',
    ]}




from typing import Optional, List, Literal, Dict
from pydantic import BaseModel, Field


# Añade modelos de pydantic para los esquemas de las herramientas aquí
# Por ejemplo:

class SumaParams(BaseModel):
    a: float = Field(..., description="Primer número a sumar")
    b: float = Field(..., description="Segundo número a sumar")



"""
TAI-CHATBOT - Main Application
"""
from contextlib import asynccontextmanager
from pathlib import Path
from fastapi import FastAPI, Depends
from fastapi.staticfiles import StaticFiles
from starlette.middleware.cors import CORSMiddleware 
from .routers import main_router, streaming_router
from .resources import setup_exception_handlers
from .auth import auth_router, get_current_user
from .llm import llm_manager

description = """
<details>
<summary>Mostrar diagrama ER</summary>

![ER](/database/.diagram.png)

</details>
"""

@asynccontextmanager
async def lifespan(app: FastAPI):
    # Inicializar LLM al iniciar la app
    await llm_manager.init()
    yield
    # Cerrar recursos al apagar la app
    await llm_manager.close()

app = FastAPI(
    title="Chatbot-API",
    version="0.1.0",
    description=description,
    lifespan=lifespan,
    redoc_url=None,
    docs_url=None
)

# Montar carpeta estática de diagramas
app.mount("/database", StaticFiles(directory=Path(__file__).parent / "database"), name="database")

# Incluir router de autenticación
app.include_router(auth_router)

app.include_router(streaming_router)

# Incluir router de API generada
app.include_router(
    main_router, 
    dependencies=[Depends(get_current_user)]
)

# Configurar manejadores de excepciones
setup_exception_handlers(app)

# Configurar CORS
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],  # Configurar según ambiente
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)


from fastapi import APIRouter, Depends, Query
from typing import Optional, List, Literal
from tai_alphi import Alphi
from datetime import datetime

from ..database import *
from ..resources import (
    APIResponse, PaginatedResponse
)


logger = Alphi.get_logger_by_name("tai-chatbot")

user_stats_router = APIRouter(
    prefix="/user-stats",
    tags=["UserStats"]
)

@user_stats_router.get("", 
    response_model=APIResponse[List[UserStatsRead]],
    response_description="Lista de registros de user_stats obtenido exitosamente",
    operation_id="user_stats_find_many",
    summary="Busca varios registros en la tabla user_stats",
    responses={
        200: {
            "description": "Lista de registros de user_stats obtenido exitosamente",
            "content": {
                "application/json": {
                    "schema": {
                        "$ref": "#/components/schemas/APIResponse_List_UserStatsRead__"
                    }
                }
            },
            "links": {
                "self": {
                    "operationId": "user_stats_find_many",
                    "description": "Enlace a la consulta actual con los mismos filtros",
                    "parameters": {
                        "username": "$request.query.username",
                        "email": "$request.query.email",
                        "total_chats": "$request.query.total_chats",
                        "active_chats": "$request.query.active_chats",
                        "total_messages": "$request.query.total_messages",
                        "created_at": "$request.query.created_at",
                        "last_activity": "$request.query.last_activity",
                        "limit": "$request.query.limit",
                        "offset": "$request.query.offset",
                        "order_by": "$request.query.order_by",
                        "order": "$request.query.order",
                        "includes": "$request.query.includes"
                    }
                },
                "item": {
                    "operationId": "user_stats_find",
                    "description": "Enlace para acceder a un elemento específico",
                    "parameters": {
                        "includes": "$request.query.includes"
                    }
                },
                "create": {
                    "operationId": "user_stats_create",
                    "description": "Enlace para crear un nuevo UserStats"
                },
                "count": {
                    "operationId": "user_stats_count",
                    "description": "Enlace para obtener el conteo total con los mismos filtros",
                    "parameters": {
                        "username": "$request.query.username",
                        "email": "$request.query.email",
                        "total_chats": "$request.query.total_chats",
                        "active_chats": "$request.query.active_chats",
                        "total_messages": "$request.query.total_messages",
                        "created_at": "$request.query.created_at",
                        "last_activity": "$request.query.last_activity",
                    }
                }}
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "El límite no puede ser negativo",
                                "field": "limit",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def user_stats_find_many(
    limit: Optional[int] = Query(None, description="Número de registros a retornar. Valores positivos toman los n primeros registros, valores negativos toman los n últimos registros (requiere order_by)", gt=0),
    order_by: List[str] = Query(None, description="Lista de nombres de columnas para ordenar los resultados.⚠️ **IMPORTANTE**: los nombres de columnas deben existir, si no serán omitidas. Requerido cuando limit es negativo"),
    order: Optional[Literal["ASC", "DESC"]] = Query("ASC", description="Dirección de ordenamiento: 'ASC' para ascendente (por defecto), 'DESC' para descendente. Solo aplica si order_by está definido", regex="^(ASC|DESC)$"),
    offset: Optional[int] = Query(None, description="Número de registros a omitir desde el inicio. Útil para paginación. Debe ser un valor no negativo", ge=0),
    username: Optional[str] = Query(None, description='Filtrar por username. ⚠️ **IMPORTANTE**: utiliza "%username%" para hacer consultas ILIKE. Campo username de la tabla user_stats', min_length=1, max_length=255),
    email: Optional[str] = Query(None, description='Filtrar por email. ⚠️ **IMPORTANTE**: utiliza "%email%" para hacer consultas ILIKE. Campo email de la tabla user_stats', min_length=1, max_length=255),
    total_chats: Optional[int] = Query(None, description="Filtrar por valor exacto de total_chats. Campo total_chats de la tabla user_stats"),
    min_total_chats: Optional[int] = Query(None, description="Valor mínimo para total_chats (incluido)"),
    max_total_chats: Optional[int] = Query(None, description="Valor máximo para total_chats (incluido)"),
    active_chats: Optional[int] = Query(None, description="Filtrar por valor exacto de active_chats. Campo active_chats de la tabla user_stats"),
    min_active_chats: Optional[int] = Query(None, description="Valor mínimo para active_chats (incluido)"),
    max_active_chats: Optional[int] = Query(None, description="Valor máximo para active_chats (incluido)"),
    total_messages: Optional[int] = Query(None, description="Filtrar por valor exacto de total_messages. Campo total_messages de la tabla user_stats"),
    min_total_messages: Optional[int] = Query(None, description="Valor mínimo para total_messages (incluido)"),
    max_total_messages: Optional[int] = Query(None, description="Valor máximo para total_messages (incluido)"),
    min_created_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_created_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    min_last_activity: Optional[datetime] = Query(None, description="Fecha y hora mínima para last_activity (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_last_activity: Optional[datetime] = Query(None, description="Fecha y hora máxima para last_activity (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    includes: List[str] = Query(None, description="Lista de relaciones a incluir en la respuesta para obtener datos relacionados. Especifica los nombres de las relaciones que deseas expandir"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[List[UserStatsRead]]:
    """
    ## Resumen
    Obtiene una lista de `user_statss` con filtros opcionales y soporte para paginación.
    
    Este endpoint permite realizar búsquedas flexibles aplicando filtros opcionales
    por cualquiera de los campos disponibles, con soporte completo para paginación
    mediante los parámetros limit y offset.

    ## Resultado
    En `APIResponse.data`, retorna un listado de objetos donde cada uno representa un registro de la tabla `user_stats` que incluye todos sus atributos

    ## Datos
    Para cada registro en `data` se incluye:
    - **username** (str): Campo username de la tabla user_stats
    - **email** (str): Campo email de la tabla user_stats
    - **total_chats** (int): Campo total_chats de la tabla user_stats
    - **active_chats** (int): Campo active_chats de la tabla user_stats
    - **total_messages** (int): Campo total_messages de la tabla user_stats
    - **created_at** (datetime): Campo created_at de la tabla user_stats
    - **last_activity** (datetime, opcional): Campo last_activity de la tabla user_stats
    
    ## Parámetros de Filtrado
    
    Todos los parámetros de filtrado son opcionales y se pueden combinar:
    - **username**: Filtrar por username. ⚠️ **IMPORTANTE**: utiliza "%username%" para hacer consultas ILIKE.
    - **email**: Filtrar por email. ⚠️ **IMPORTANTE**: utiliza "%email%" para hacer consultas ILIKE.
    - **total_chats**: Filtrar por total_chats
    - **min_total_chats**: Filtrar por fecha mínima (incluída)
    - **max_total_chats**: Filtrar por fecha máxima (incluída)
    - **active_chats**: Filtrar por active_chats
    - **min_active_chats**: Filtrar por fecha mínima (incluída)
    - **max_active_chats**: Filtrar por fecha máxima (incluída)
    - **total_messages**: Filtrar por total_messages
    - **min_total_messages**: Filtrar por fecha mínima (incluída)
    - **max_total_messages**: Filtrar por fecha máxima (incluída)
    - **min_created_at**: Filtrar por valor mínimo de created_at (incluído el valor del filtro)
    - **max_created_at**: Filtrar por valor máximo de created_at (incluído el valor del filtro)
    - **min_last_activity**: Filtrar por valor mínimo de last_activity (incluído el valor del filtro)
    - **max_last_activity**: Filtrar por valor máximo de last_activity (incluído el valor del filtro)

    
    ## Parámetros de Paginación
    
    - **limit**: Número máximo de registros a retornar. Solo admite valores positivos. Si no se especifica, retorna todos los registros que coincidan con los filtros.
    - **order_by**: Lista de nombres de columnas para ordenar los resultados.⚠️ **IMPORTANTE**: los nombres de columnas deben existir, si no serán omitidas.
    - **order**: Dirección de ordenamiento: 'ASC' para ascendente (por defecto), 'DESC' para descendente. Solo aplica si order_by está definido.
    - **offset**: Número de registros a omitir desde el inicio. Solo admite valores positivos. Si no se especifica, inicia desde el primer registro.
    
    ## Consulta combinada (recomendado para pocos registros)
    ⚠️ **IMPORTANTE**: Usa siempre el parámetro `includes` para cargar relaciones en una sola consulta y evitar múltiples llamadas al API.
    
    ⚠️ **WARNING**: Si la relación incluida tiene muchos registros relacionados, la respuesta puede ser muy grande y lenta. Mejor consultar su endpoint directamente con filtros.
    
    El parametro `includes` permite cargar relaciones asociadas a los registros.
    """
    result = await api.user_stats.find_many(
        limit=limit,
        offset=offset,
        order_by=order_by,
        order=order,
        username=username,
        email=email,
        total_chats=total_chats,
        min_total_chats=min_total_chats,
        max_total_chats=max_total_chats,
        active_chats=active_chats,
        min_active_chats=min_active_chats,
        max_active_chats=max_active_chats,
        total_messages=total_messages,
        min_total_messages=min_total_messages,
        max_total_messages=max_total_messages,
        min_created_at=min_created_at,
        max_created_at=max_created_at,
        min_last_activity=min_last_activity,
        max_last_activity=max_last_activity,
        includes=includes
    )
    
    # Obtener el total para metadatos de paginación si es necesario
    total = None
    if limit is not None or offset is not None:
        try:
            total = await api.user_stats.count(
                username=username,
                email=email,
                total_chats=total_chats,
                min_total_chats=min_total_chats,
                max_total_chats=max_total_chats,
                active_chats=active_chats,
                min_active_chats=min_active_chats,
                max_active_chats=max_active_chats,
                total_messages=total_messages,
                min_total_messages=min_total_messages,
                max_total_messages=max_total_messages,
                min_created_at=min_created_at,
                max_created_at=max_created_at,
                min_last_activity=min_last_activity,
                max_last_activity=max_last_activity,
            )
        except Exception as e:
            logger.warning(f"No se pudo obtener el total de registros: {str(e)}")
    
    return PaginatedResponse.success_paginated(
        data=result,
        total=total,
        limit=limit,
        offset=offset,
        message=f"UserStatss obtenidos exitosamente"
    )




from fastapi import APIRouter
from .router_usuario import usuario_router
from .router_chat import chat_router
from .router_mensaje import mensaje_router
from .router_token_usage import token_usage_router
from .router_user_stats import user_stats_router
from .router_token_consumption_stats import token_consumption_stats_router
from .router_chat_activity import chat_activity_router
from .router_enums import enumerations_router
from .router_stream import streaming_router

main_router = APIRouter()

main_router.include_router(usuario_router)
main_router.include_router(chat_router)
main_router.include_router(mensaje_router)
main_router.include_router(token_usage_router)
main_router.include_router(user_stats_router)
main_router.include_router(token_consumption_stats_router)
main_router.include_router(chat_activity_router)
main_router.include_router(enumerations_router)

__all__ = [
    "main_router",
    "streaming_router"
]



# -*- coding: utf-8 -*-
"""
Fuente principal para la definición de esquemas y generación de modelos CRUD.
Usa el contenido de tai_sql para definir tablas, relaciones, vistas y generar automáticamente modelos y CRUDs.
Usa tai_sql.generators para generar modelos y CRUDs basados en las tablas definidas.
Ejecuta por consola tai_sql generate para generar los recursos definidos en este esquema.
"""
from __future__ import annotations
from tai_sql import *
from tai_sql.generators import *
from database.custom_generators import *

# Configurar el datasource
datasource(
    provider=env('CHATBOT_DATABASE_URL'), # Además de env, también puedes usar (para testing) connection_string y params
    schema='chatbot', # Esquema del datasource
)

# Configurar los generadores
generate(
    CustomModelsGenerator(
        output_dir='tai_chat/api/database' # Directorio donde se generarán los modelos
    ),
    ModelsGenerator('tai_chat/cli/commands/cmd_adduser'),
    CustomAsyncCRUDGenerator(
        output_dir='tai_chat/api/database',
        logger_name='tai-chatbot'
    ),
    CRUDGenerator('tai_chat/cli/commands/cmd_adduser', logger_name='tai-chatbot', models_import_path='...models'),
    CustomERDiagramGenerator(
        output_dir='tai_chat/api/database', # Directorio donde se generarán los diagramas
    )
)

# Definición de tablas y relaciones para TAI-CHAT

class Usuario(Table):
    """Tabla que almacena información de los usuarios del chatbot"""
    __tablename__ = "usuario"

    username: str = column(primary_key=True, description='Nombre de usuario único')
    password: str = column(encrypt=True, description='Contraseña encriptada')
    email: Optional[str] = column(description='Correo electrónico del usuario')
    avatar: Optional[str] = column(description='URL del avatar del usuario')
    session_id: Optional[str] = column(description='ID de la sesión activa del usuario')
    created_at: datetime = column(default=datetime.now, description='Fecha de creación del usuario')
    updated_at: datetime = column(default=datetime.now, description='Fecha de última actualización')
    is_active: bool = column(default=True, description='Estado activo del usuario')

    chats: List[Chat] # Relación one-to-many con la tabla Chat


class MessageRole(Enum):
    """Roles de los mensajes en el chat"""
    USER = 'usuario'
    ASSISTANT = 'asistente'
    SYSTEM = 'sistema'


class Chat(Table):
    """Tabla que almacena las conversaciones del chatbot"""
    __tablename__ = "chat"

    id: bigint = column(primary_key=True, autoincrement=True, description='UUID del chat')
    title: str = column(description='Título de la conversación')
    username: str = column(description='ID del usuario propietario del chat')
    created_at: datetime = column(default=datetime.now, description='Fecha de creación del chat')
    updated_at: datetime = column(default=datetime.now, description='Fecha de última actualización')
    is_active: bool = column(default=True, description='Estado activo del chat')

    messages: List[Mensaje] # Relación one-to-many con la tabla Mensaje
    
    usuario: Usuario = relation(fields=['username'], references=['username'], backref='chats') # Relación many-to-one con Usuario


class Mensaje(Table):
    """Tabla que almacena los mensajes individuales de cada chat"""
    __tablename__ = "mensaje"

    id: bigint = column(primary_key=True, autoincrement=True, description='UUID del mensaje')
    content: str = column(description='Contenido del mensaje')
    role: MessageRole = column(description='Rol del mensaje (user, assistant, system)')
    timestamp: datetime = column(default=datetime.now, description='Timestamp del mensaje')
    chat_id: bigint = column(description='ID del chat al que pertenece el mensaje')

    token_usage: List[TokenUsage] # Relación one-to-many con la tabla TokenUsage

    chat: Chat = relation(fields=['chat_id'], references=['id'], backref='messages') # Relación many-to-one con Chat


class TokenUsage(Table):
    """Tabla que almacena el consumo de tokens para métricas y facturación"""
    __tablename__ = "token_usage"

    id: bigint = column(primary_key=True, autoincrement=True)
    prompt_tokens: int = column(default=0, description='Tokens consumidos en el prompt')
    completion_tokens: int = column(default=0, description='Tokens consumidos en la respuesta')
    total_tokens: int = column(default=0, description='Total de tokens consumidos')
    model_name: str = column(description='Nombre del modelo utilizado')
    provider: str = column(description='Proveedor del modelo (OpenAI, Anthropic, etc.)')
    cost_usd: Optional[float] = column(description='Costo estimado en USD')
    timestamp: datetime = column(default=datetime.now, description='Timestamp del consumo')
    message_id: bigint = column(description='ID del mensaje asociado')

    message: Mensaje = relation(fields=['message_id'], references=['id'], backref='token_usage') # Relación one-to-one con Mensaje


# Definición de vistas

class UserStats(View):
    """Vista que muestra estadísticas de usuarios y sus chats"""
    __tablename__ = "user_stats"
    __query__ = query('user_stats.sql') # Esto es necesario para usar tai-sql push

    username: str
    email: str
    total_chats: int
    active_chats: int
    total_messages: int
    created_at: datetime
    last_activity: Optional[datetime]


class TokenConsumptionStats(View):
    """Vista que muestra estadísticas de consumo de tokens por usuario y período"""
    __tablename__ = "token_consumption_stats"
    __query__ = query('token_consumption_stats.sql')

    username: str
    date: datetime
    total_prompt_tokens: int
    total_completion_tokens: int
    total_tokens: int
    total_cost_usd: Optional[float]
    chat_count: int
    most_used_model: str
    most_used_provider: str


class ChatActivity(View):
    """Vista que muestra la actividad reciente de chats"""
    __tablename__ = "chat_activity"
    __query__ = query('chat_activity.sql')

    chat_id: str
    chat_title: str
    username: str
    message_count: int
    last_message_timestamp: datetime
    total_tokens_consumed: int
    is_active: bool
    


"""
chat-demo - Authentication Endpoints
Generado automáticamente por tai-api set-auth

Este módulo contiene los endpoints de login y logout con manejo de sesiones.
"""

from fastapi import APIRouter, Depends
from fastapi.security import OAuth2PasswordRequestForm
from pydantic import BaseModel

from .jwt import JWTHandler
from .dependencies import get_current_user, CurrentUser
from ..database import ChatbotAsyncDBAPI, UsuarioUpdateValues
from ..resources import (
    APIResponse,
    InvalidCredentialsException,
    ConcurrentSessionDetectedException,
    DatabaseException
)

router = APIRouter(prefix="/auth", tags=["Autenticación"])

class LoginRequest(BaseModel):
    """Modelo para datos de login"""
    username: str
    pwd: str

class LoginData(BaseModel):
    """Modelo para datos de respuesta de login"""
    access_token: str
    token_type: str = "bearer"
    user: dict  # Información básica del usuario
    session_id: str

class LogoutData(BaseModel):
    """Modelo para datos de respuesta de logout"""
    message: str = "Sesión cerrada exitosamente"

class UserInfoData(BaseModel):
    """Modelo para datos de información de usuario"""
    username: str


async def authenticate(
    username: str,
    pwd: str,
    api: ChatbotAsyncDBAPI,
    from_swagger: bool=False
) -> APIResponse[LoginData] | LoginData:
    """
    Función auxiliar que realiza la lógica de login compartida.
    
    Args:
        username: Nombre de usuario
        password: Contraseña
        api: DAO para acceso a datos
        
    Returns:
        APIResponse[LoginData]: Respuesta estándar con token y datos de usuario
        
    Raises:
        InvalidCredentialsException: Si credenciales son incorrectas
        DatabaseException: Si hay error en BD
    """
    try:
        # 1. Buscar usuario
        user = await api.usuario.find(
            username=username
        )
        
        if not user:
            raise InvalidCredentialsException()
        
        # 2. Verificar contraseña
        if user.password != pwd:
            raise InvalidCredentialsException()
        
        # 3. Generar nuevo session_id
        new_session_id = JWTHandler.generate_session_id()
        
        # 4. Actualizar session_id en BD
        await api.usuario.update(
            username=user.username,
            updated_values=UsuarioUpdateValues(
                session_id=new_session_id
            )
        )
        
        # Refrescar el objeto user
        user = await api.usuario.find(username=user.username)
        
        # 5. Generar token JWT
        token = JWTHandler.create_token(
            username=username,
            session_id=new_session_id
        )
        
        # 6. Preparar respuesta (sin datos sensibles)
        user_data = {
            "username": user.username,
            # Agregar otros campos públicos según necesites
        }
        
        login_data = LoginData(
            access_token=token,
            user=user_data,
            session_id=new_session_id
        )

        if from_swagger:
            return login_data
        
        return APIResponse.success(
            data=login_data,
            message="Autenticación exitosa"
        )
        
    except Exception as e:
        if isinstance(e, (InvalidCredentialsException, ConcurrentSessionDetectedException)):
            raise e
        raise DatabaseException(f"Error en login: {str(e)}")


@router.post("/login", response_model=APIResponse[LoginData])
async def login(
    credentials: LoginRequest,
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
):
    """
    Endpoint de autenticación de usuarios
    """
    return await authenticate(credentials.username, credentials.pwd, api)


@router.post("/docslogin", response_model=LoginData, include_in_schema=False)
async def docs_login(
    form_data: OAuth2PasswordRequestForm = Depends(),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
):
    """
    Endpoint de autenticación para la documentación de FastAPI (formulario)
    """
    return await authenticate(form_data.username, form_data.password, api, from_swagger=True)


@router.post("/logout", response_model=APIResponse[LogoutData])
async def logout(
    current_user: CurrentUser = Depends(get_current_user),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
):
    """
    Endpoint para cerrar sesión.
    Invalida la sesión actual eliminando o cambiando el session_id
    
    Args:
        current_user: Usuario autenticado (obtenido de token)
        
    Returns:
        APIResponse[LogoutData]: Respuesta estándar de confirmación de logout
        
    Raises:
        DatabaseException: Si hay error al invalidar sesión
    """
    
    try:
        # Invalidar sesión actual
        await api.usuario.update(
            username=current_user.username,
            updated_values=UsuarioUpdateValues(
                session_id=None
            )
        )
        
        logout_data = LogoutData()
        return APIResponse.success(
            data=logout_data,
            message="Sesión cerrada exitosamente"
        )
        
    except Exception as e:
        raise DatabaseException(f"Error en logout: {str(e)}")


@router.get("/me", response_model=APIResponse[UserInfoData])
async def get_current_user_info(current_user: CurrentUser = Depends(get_current_user)):
    """
    Endpoint para obtener información del usuario actual.
    
    Útil para verificar si el token sigue siendo válido
    y obtener datos actualizados del usuario.
    
    Args:
        current_user: Usuario autenticado
        
    Returns:
        APIResponse[UserInfoData]: Respuesta estándar con información pública del usuario
    """
    
    user_info = UserInfoData(
        username=current_user.username,
    )
    
    return APIResponse.success(
        data=user_info,
        message="Información del usuario obtenida exitosamente"
    )


from typing import Dict, Any
from datetime import datetime
from langchain_core.messages import BaseMessage
from tai_alphi import Alphi

from .database import (
    chatbot_api,
    MensajeCreate,
    MensajeRead,
    ChatUpdateValues
)

logger = Alphi.get_logger_by_name("tai-chatbot")

async def add_message(
    chat_id: str, 
    msg: BaseMessage
) -> Dict[str, bool | MensajeRead | str]:
    """
    Añadir mensaje al contexto y guardarlo en BD
    
    Args:
        chat_id: ID del chat
        msg: Mensaje a añadir (HumanMessage o AIMessage)
        
    Returns:
        Dict: Información del mensaje creado
    """
    try:
        
        # Crear mensaje en BD
        message = await chatbot_api.mensaje.create(MensajeCreate(content=msg.content, role=msg.type, chat_id=chat_id))
        
        # Actualizar timestamp del chat
        await chatbot_api.chat.update(chat_id, updated_values=ChatUpdateValues(updated_at=datetime.now()))
        
        return {
            "msg": message,
            "success": True
        }
        
    except Exception as e:
        logger.error(f"[chatbot] Error añadiendo mensaje al chat {chat_id}: {e}")
        return {"success": False, "error": str(e)}


from fastapi import APIRouter, Depends, Query, Path, Body
from typing import Optional, List, Literal
from tai_alphi import Alphi
from datetime import datetime

from ..database import *
from ..resources import (
    APIResponse, PaginatedResponse, RecordNotFoundException,
    ValidationException
)


logger = Alphi.get_logger_by_name("tai-chatbot")

mensaje_router = APIRouter(
    prefix="/mensaje",
    tags=["Mensaje"]
)

@mensaje_router.get("", 
    response_model=APIResponse[List[MensajeRead]],
    response_description="Lista de registros de mensaje obtenido exitosamente",
    operation_id="mensaje_find_many",
    summary="Busca varios registros en la tabla mensaje",
    responses={
        200: {
            "description": "Lista de registros de mensaje obtenido exitosamente",
            "content": {
                "application/json": {
                    "schema": {
                        "$ref": "#/components/schemas/APIResponse_List_MensajeRead__"
                    }
                }
            },
            "links": {
                "self": {
                    "operationId": "mensaje_find_many",
                    "description": "Enlace a la consulta actual con los mismos filtros",
                    "parameters": {
                        "content": "$request.query.content",
                        "role": "$request.query.role",
                        "timestamp": "$request.query.timestamp",
                        "chat_id": "$request.query.chat_id",
                        "limit": "$request.query.limit",
                        "offset": "$request.query.offset",
                        "order_by": "$request.query.order_by",
                        "order": "$request.query.order",
                        "includes": "$request.query.includes"
                    }
                },
                "item": {
                    "operationId": "mensaje_find",
                    "description": "Enlace para acceder a un elemento específico",
                    "parameters": {
                        "id": "$response.body#/data/**/id",
                        "includes": "$request.query.includes"
                    }
                },
                "create": {
                    "operationId": "mensaje_create",
                    "description": "Enlace para crear un nuevo Mensaje"
                },
                "count": {
                    "operationId": "mensaje_count",
                    "description": "Enlace para obtener el conteo total con los mismos filtros",
                    "parameters": {
                        "content": "$request.query.content",
                        "role": "$request.query.role",
                        "timestamp": "$request.query.timestamp",
                        "chat_id": "$request.query.chat_id",
                    }
                },
            "token_usage": {
            "operationId": "tokenusage_find_many",
                    "description": "Enlace a los TokenUsages relacionados",
                    "parameters": {
                        "id": "$response.body#/data/**/message_id",
                        "includes": "$request.query.includes"
                    }
            },
            "chat": {
            "operationId": "chat_find",
                    "description": "Enlace al Chat relacionado",
                    "parameters": {
                        "id": "$response.body#/data/**/chat_id",
                        "includes": "$request.query.includes"
                    }
            }
            }
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "El límite no puede ser negativo",
                                "field": "limit",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_find_many(
    limit: Optional[int] = Query(None, description="Número de registros a retornar. Valores positivos toman los n primeros registros, valores negativos toman los n últimos registros (requiere order_by)", gt=0),
    order_by: List[str] = Query(None, description="Lista de nombres de columnas para ordenar los resultados.⚠️ **IMPORTANTE**: los nombres de columnas deben existir, si no serán omitidas. Requerido cuando limit es negativo"),
    order: Optional[Literal["ASC", "DESC"]] = Query("ASC", description="Dirección de ordenamiento: 'ASC' para ascendente (por defecto), 'DESC' para descendente. Solo aplica si order_by está definido", regex="^(ASC|DESC)$"),
    offset: Optional[int] = Query(None, description="Número de registros a omitir desde el inicio. Útil para paginación. Debe ser un valor no negativo", ge=0),
    content: Optional[str] = Query(None, description='Filtrar por content. ⚠️ **IMPORTANTE**: utiliza "%content%" para hacer consultas ILIKE. Contenido del mensaje', min_length=1, max_length=255),
    role: Optional[str] = Query(None, description='Filtrar por role. ⚠️ **IMPORTANTE**: utiliza "%role%" para hacer consultas ILIKE. Rol del mensaje (user, assistant, system)', min_length=1, max_length=255),
    min_timestamp: Optional[datetime] = Query(None, description="Fecha y hora mínima para timestamp (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_timestamp: Optional[datetime] = Query(None, description="Fecha y hora máxima para timestamp (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    chat_id: Optional[int] = Query(None, description="Filtrar por chat_id (clave foránea)"),
    includes: List[str] = Query(None, description="Lista de relaciones a incluir en la respuesta para obtener datos relacionados. Especifica los nombres de las relaciones que deseas expandir"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[List[MensajeRead]]:
    """
    ## Resumen
    Obtiene una lista de `mensajes` con filtros opcionales y soporte para paginación.
    
    Este endpoint permite realizar búsquedas flexibles aplicando filtros opcionales
    por cualquiera de los campos disponibles, con soporte completo para paginación
    mediante los parámetros limit y offset.

    ## Resultado
    En `APIResponse.data`, retorna un listado de objetos donde cada uno representa un registro de la tabla `mensaje` que incluye todos sus atributos

    ## Datos
    Para cada registro en `data` se incluye:
    - **id** (int): UUID del mensaje
    - **content** (str): Contenido del mensaje
    - **role** (str): Rol del mensaje (user, assistant, system)
    - **timestamp** (datetime): Timestamp del mensaje
    - **chat_id** (int): ID del chat al que pertenece el mensaje
    
    ## Parámetros de Filtrado
    
    Todos los parámetros de filtrado son opcionales y se pueden combinar:
    - **content**: Filtrar por content. ⚠️ **IMPORTANTE**: utiliza "%content%" para hacer consultas ILIKE.
    - **role**: Filtrar por role. ⚠️ **IMPORTANTE**: utiliza "%role%" para hacer consultas ILIKE.
    - **min_timestamp**: Filtrar por valor mínimo de timestamp (incluído el valor del filtro)
    - **max_timestamp**: Filtrar por valor máximo de timestamp (incluído el valor del filtro)
    - **chat_id**: Filtrar por chat_id

    
    ## Parámetros de Paginación
    
    - **limit**: Número máximo de registros a retornar. Solo admite valores positivos. Si no se especifica, retorna todos los registros que coincidan con los filtros.
    - **order_by**: Lista de nombres de columnas para ordenar los resultados.⚠️ **IMPORTANTE**: los nombres de columnas deben existir, si no serán omitidas.
    - **order**: Dirección de ordenamiento: 'ASC' para ascendente (por defecto), 'DESC' para descendente. Solo aplica si order_by está definido.
    - **offset**: Número de registros a omitir desde el inicio. Solo admite valores positivos. Si no se especifica, inicia desde el primer registro.
    
    ## Consulta combinada (recomendado para pocos registros)
    ⚠️ **IMPORTANTE**: Usa siempre el parámetro `includes` para cargar relaciones en una sola consulta y evitar múltiples llamadas al API.
    
    ⚠️ **WARNING**: Si la relación incluida tiene muchos registros relacionados, la respuesta puede ser muy grande y lenta. Mejor consultar su endpoint directamente con filtros.
    
    El parametro `includes` permite cargar relaciones asociadas a los registros.

    ### Relaciones disponibles
    - **token_usage**: lista de TokenUsage relacionados (one-to-many)

        - **descripción**: Tabla que almacena el consumo de tokens para métricas y facturación
    - **chat**: Chat relacionado (many-to-one)

        - **descripción**: Tabla que almacena las conversaciones del chatbot
    
    ### Ejemplos básicos:
    #### Solo datos básicos
    `mensaje = GET /mensaje`
    
    #### Incluir token_usage
    `mensaje = GET /mensaje?includes=token_usage`
    
    #### Incluir chat
    `mensaje = GET /mensaje?includes=chat`
    
    #### Múltiples relaciones en una sola consulta
    `mensaje = GET /mensaje?includes=token_usage&includes=chat`
    
    #### Relaciones anidadas
    Puedes incluir los datos de token_usage y además incluir sus propias relaciones  
    `mensaje = GET /mensaje?includes=token_usage.{nested_relation}`  
    Puedes incluir los datos de chat y además incluir sus propias relaciones  
    `mensaje = GET /mensaje?includes=chat.{nested_relation}`  
    """
    result = await api.mensaje.find_many(
        limit=limit,
        offset=offset,
        order_by=order_by,
        order=order,
        content=content,
        role=role,
        min_timestamp=min_timestamp,
        max_timestamp=max_timestamp,
        chat_id=chat_id,
        includes=includes
    )
    
    # Obtener el total para metadatos de paginación si es necesario
    total = None
    if limit is not None or offset is not None:
        try:
            total = await api.mensaje.count(
                content=content,
                role=role,
                min_timestamp=min_timestamp,
                max_timestamp=max_timestamp,
                chat_id=chat_id,
            )
        except Exception as e:
            logger.warning(f"No se pudo obtener el total de registros: {str(e)}")
    
    return PaginatedResponse.success_paginated(
        data=result,
        total=total,
        limit=limit,
        offset=offset,
        message=f"Mensajes obtenidos exitosamente"
    )

@mensaje_router.get("/{id:int}", 
    response_model=APIResponse[MensajeRead],
    response_description="Registro único de mensaje obtenido exitosamente",
    operation_id="mensaje_find",
    summary="Busca un registro en la tabla mensaje",
    responses={
        200: {
            "description": "Registro único de mensaje obtenido exitosamente",
            "content": {
                "application/json": {
                    "schema": {
                        "$ref": "#/components/schemas/APIResponse_MensajeRead_"
                    }
                }
            },
            "links": {
                "self": {
                    "operationId": "mensaje_find",
                    "description": "Enlace al recurso actual",
                    "parameters": {
                        "id": "$response.body#/data/id",
                        "includes": "$request.query.includes"
                    }
                },
                "collection": {
                    "operationId": "mensaje_find_many",
                    "description": "Enlace a la colección de Mensajes"
                },
                "edit": {
                    "operationId": "mensaje_update",
                    "description": "Enlace para actualizar este Mensaje",
                    "parameters": {
                        "id": "$response.body#/data/id",
                    }
                },
                "delete": {
                    "operationId": "mensaje_delete",
                    "description": "Enlace para eliminar este Mensaje",
                    "parameters": {
                        "id": "$response.body#/data/id",
                    }
                },
            "token_usage": {
            "operationId": "tokenusage_find_many",
                    "description": "Enlace a los TokenUsages relacionados",
                    "parameters": {
                        "id": "$response.body#/data/message_id",
                        "includes": "$request.query.includes"
                    }
            },
            "chat": {
            "operationId": "chat_find",
                    "description": "Enlace al Chat relacionado",
                    "parameters": {
                        "id": "$response.body#/data/chat_id",
                        "includes": "$request.query.includes"
                    }
            }
            }
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "id debe ser mayor a 0",
                                "field": "id",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        404: {
            "description": "Mensaje no encontrado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Mensaje no encontrado",
                        "errors": [
                            {
                                "code": "RECORD_NOT_FOUND",
                                "message": "Mensaje no encontrado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_find(
    id: int = Path(..., description="UUID del mensaje", gt=0),
    includes: List[str] = Query(None, description="Lista de relaciones a incluir en la respuesta para obtener datos relacionados. Especifica los nombres de las relaciones que deseas expandir"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[MensajeRead]:
    """
    ## Resumen
    Obtiene un Mensaje específico por su clave primaria.
    
    Este endpoint permite recuperar un registro individual de Mensaje
    utilizando su identificador único (clave primaria). Opcionalmente puede
    incluir datos de relaciones asociadas.

    ## Resultado
    Si la consulta es exitosa, en `APIResponse.data`, retorna un objeto que representa un registro de la tabla `mensaje` que incluye todos sus atributos

    Si no se encuentra el registro, devuelve un error 404 `RECORD_NOT_FOUND`.

    ## Datos
    Para cada registro en `data` se incluye:
    - **id** (int): UUID del mensaje
    - **content** (str): Contenido del mensaje
    - **role** (str): Rol del mensaje (user, assistant, system)
    - **timestamp** (datetime): Timestamp del mensaje
    - **chat_id** (int): ID del chat al que pertenece el mensaje
    
    ## Parámetros de Identificación
    
    - **id**: id del Mensaje a buscar (tipo: int)
    
    ## Consulta combinada (RECOMENDADO)
    ⚠️ **IMPORTANTE**: Usa siempre el parámetro `includes` para cargar relaciones en una sola consulta y evitar múltiples llamadas al API.
    
    El parametro `includes` permite cargar relaciones asociadas a los registros.

    ### Relaciones disponibles (usar con parámetro 'includes'):
    - token_usage: Lista de TokenUsage relacionados (one-to-many)
        Tabla que almacena el consumo de tokens para métricas y facturación
    - chat: Chat relacionado (many-to-one)
        Tabla que almacena las conversaciones del chatbot
    
    ### Uso del parámetro 'includes':
    Para cargar relaciones específicas, usa el parámetro 'includes' en la consulta:
    
    ### Ejemplos básicos:
    #### Solo datos básicos
    `mensaje = GET /mensaje/{id:int}`
    
    #### Incluir token_usage
    `mensaje = GET /mensaje/{id:int}?includes=token_usage`
    
    #### Incluir chat
    `mensaje = GET /mensaje/{id:int}?includes=chat`
    
    #### Múltiples relaciones en una sola consulta
    `mensaje = GET /mensaje/{id:int}?includes=token_usage&includes=chat`
    
    #### Relaciones anidadas
    Puedes incluir los datos de token_usage y además incluir sus propias relaciones  
    `mensaje = GET /mensaje/{id:int}?includes=token_usage.{nested_relation}`
    Puedes incluir los datos de chat y además incluir sus propias relaciones  
    `mensaje = GET /mensaje/{id:int}?includes=chat.{nested_relation}`
    """
    # Validaciones básicas de entrada
    if id <= 0:
        raise ValidationException("id debe ser mayor a 0", "id")
    
    result = await api.mensaje.find(
        id=id,
        includes=includes
    )
    
    if result is None:
        raise RecordNotFoundException("Mensaje")
        
    return APIResponse.success(
        data=result,
        message="Mensaje obtenido exitosamente"
    )

@mensaje_router.get("/count", 
    response_model=APIResponse[int],
    response_description="Número de registros de mensaje según los filtros aplicados",
    operation_id="mensaje_count",
    summary="Cuenta registros en la tabla mensaje",
    responses={
        200: {
            "description": "Conteo realizado exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": 42,
                        "message": "Conteo realizado exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_count(
    content: Optional[str] = Query(None, description='Filtrar por content. ⚠️ **IMPORTANTE**: utiliza "%content%" para hacer consultas ILIKE. Contenido del mensaje', min_length=1, max_length=255),
    role: Optional[str] = Query(None, description='Filtrar por role. ⚠️ **IMPORTANTE**: utiliza "%role%" para hacer consultas ILIKE. Rol del mensaje (user, assistant, system)', min_length=1, max_length=255),
    min_timestamp: Optional[datetime] = Query(None, description="Fecha y hora mínima para timestamp (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_timestamp: Optional[datetime] = Query(None, description="Fecha y hora máxima para timestamp (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    chat_id: Optional[int] = Query(None, description="Filtrar por chat_id (clave foránea)"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Cuenta el número de Mensajes que coinciden con los filtros.
    """
    result = await api.mensaje.count(
        content=content,
        role=role,
        min_timestamp=min_timestamp,
        max_timestamp=max_timestamp,
        chat_id=chat_id,
    )
    
    return APIResponse.success(
        data=result,
        message="Conteo realizado exitosamente"
    )

@mensaje_router.get("/exists", 
    response_model=APIResponse[bool],
    operation_id="mensaje_exists",
    summary="Verifica existencia en la tabla mensaje",
    responses={
        200: {
            "description": "Verificación realizada exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": True,
                        "message": "Verificación realizada exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_exists(
    content: Optional[str] = Query(None, description='Filtrar por content. ⚠️ **IMPORTANTE**: utiliza "%content%" para hacer consultas ILIKE. Contenido del mensaje', min_length=1, max_length=255),
    role: Optional[str] = Query(None, description='Filtrar por role. ⚠️ **IMPORTANTE**: utiliza "%role%" para hacer consultas ILIKE. Rol del mensaje (user, assistant, system)', min_length=1, max_length=255),
    min_timestamp: Optional[datetime] = Query(None, description="Fecha y hora mínima para timestamp (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_timestamp: Optional[datetime] = Query(None, description="Fecha y hora máxima para timestamp (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    chat_id: Optional[int] = Query(None, description="Filtrar por chat_id (clave foránea)"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[bool]:
    """
    Verifica si existe al menos un mensaje que coincida con los filtros.
    """
    result = await api.mensaje.exists(
        content=content,
        role=role,
        min_timestamp=min_timestamp,
        max_timestamp=max_timestamp,
        chat_id=chat_id,
    )
    
    return APIResponse.success(
        data=result,
        message="Verificación realizada exitosamente"
    )

@mensaje_router.post("", 
    response_model=APIResponse[MensajeRead],
    status_code=201,
    operation_id="mensaje_create",
    summary="Crea un registro en la tabla mensaje",
    responses={
        422: {
            "description": "Error de validación en los datos de entrada",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "El campo es requerido",
                                "field": "content",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        409: {
            "description": "Registro duplicado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "El registro ya existe",
                        "errors": [
                            {
                                "code": "DUPLICATE_RECORD",
                                "message": "El registro ya existe",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Violación de clave foránea",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Violación de clave foránea",
                        "errors": [
                            {
                                "code": "FOREIGN_KEY_VIOLATION",
                                "message": "La referencia especificada no existe",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_create(
    mensaje: MensajeCreate,
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[MensajeRead]:
    """
    Crea un nuevo Mensaje.
    """
    result = await api.mensaje.create(mensaje)
    
    return APIResponse.success(
        data=result,
        message="Mensaje creado exitosamente"
    )

@mensaje_router.patch("/{id:int}", 
    response_model=APIResponse[int],
    operation_id="mensaje_update",
    summary="Actualiza un registro en la tabla mensaje",
    responses={
        200: {
            "description": "Mensaje actualizado exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": 1,
                        "message": "Mensaje actualizado exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Error de validación en parámetros o datos",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "id debe ser mayor a 0",
                                "field": "id",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        404: {
            "description": "Mensaje no encontrado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Mensaje no encontrado",
                        "errors": [
                            {
                                "code": "RECORD_NOT_FOUND",
                                "message": "Mensaje no encontrado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Violación de clave foránea",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Violación de clave foránea",
                        "errors": [
                            {
                                "code": "FOREIGN_KEY_VIOLATION",
                                "message": "La referencia especificada no existe",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_update(
    id: int = Path(..., description="UUID del mensaje", gt=0),
    values: MensajeUpdateValues = Body(...),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Actualiza un Mensaje específico.
    """
    # Validaciones básicas de entrada
    if id <= 0:
        raise ValidationException("id debe ser mayor a 0", "id")
    
    # Verificar que el registro existe antes de actualizar
    existing = await api.mensaje.find(
        id=id,
    )
    
    if existing is None:
        raise RecordNotFoundException("Mensaje")
    
    result = await api.mensaje.update(
        id=id,
        updated_values=values
    )
    
    if result == 0:
        raise RecordNotFoundException("Mensaje")
        
    return APIResponse.success(
        data=result,
        message="Mensaje actualizado exitosamente"
    )

@mensaje_router.patch("", 
    response_model=APIResponse[int],
    operation_id="mensaje_update_many",
    summary="Actualiza múltiples registros en la tabla mensaje",
    responses={
        200: {
            "description": "Mensajes actualizados exitosamente",
            "content": {
                "application/json": {
                    "examples": {
                        "records_updated": {
                            "summary": "Registros actualizados",
                            "value": {
                                "status": "success",
                                "data": 5,
                                "message": "5 Mensajes actualizados exitosamente",
                                "errors": None,
                                "meta": None
                            }
                        },
                        "no_records_found": {
                            "summary": "No se encontraron registros",
                            "value": {
                                "status": "success",
                                "data": 0,
                                "message": "No se encontraron registros que coincidan con los criterios",
                                "errors": None,
                                "meta": None
                            }
                        }
                    }
                }
            }
        },
        422: {
            "description": "Error de validación en los datos",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "Los criterios de búsqueda son requeridos",
                                "field": "filters",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_update_many(
    payload: MensajeUpdate,
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Actualiza múltiples Mensajes.
    """
    result = await api.mensaje.update_many(payload)
    
    message = f"{result} Mensajes actualizados exitosamente" if result > 0 else "No se encontraron registros que coincidan con los criterios"
    
    return APIResponse.success(
        data=result,
        message=message
    )

@mensaje_router.delete("/{id:int}", 
    response_model=APIResponse[int],
    operation_id="mensaje_delete",
    summary="Elimina un registro en la tabla mensaje",
    responses={
        200: {
            "description": "Mensaje eliminado exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": 1,
                        "message": "Mensaje eliminado exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "id debe ser mayor a 0",
                                "field": "id",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        404: {
            "description": "Mensaje no encontrado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Mensaje no encontrado",
                        "errors": [
                            {
                                "code": "RECORD_NOT_FOUND",
                                "message": "Mensaje no encontrado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Violación de clave foránea",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Violación de clave foránea",
                        "errors": [
                            {
                                "code": "FOREIGN_KEY_VIOLATION",
                                "message": "No se puede eliminar el registro porque está siendo referenciado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def mensaje_delete(
    id: int = Path(..., description="UUID del mensaje", gt=0),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Elimina un Mensaje por su primary key.
    """
    # Validaciones básicas de entrada
    if id <= 0:
        raise ValidationException("id debe ser mayor a 0", "id")
    
    # Verificar que el registro existe antes de eliminar
    existing = await api.mensaje.find(
        id=id,
    )
    
    if existing is None:
        raise RecordNotFoundException("Mensaje")
    
    result = await api.mensaje.delete(
        id=id,
    )
    
    if result == 0:
        raise RecordNotFoundException("Mensaje")
        
    return APIResponse.success(
        data=result,
        message="Mensaje eliminado exitosamente"
    )



"""
Excepciones personalizadas para tai-api.

Este módulo define excepciones específicas que se pueden lanzar en los endpoints
y que serán manejadas por el sistema de respuestas.
"""

from typing import Any, Optional, Dict
from enum import Enum

class ErrorCode(str, Enum):
    """Códigos de error estandardizados."""
    # Errores de validación
    VALIDATION_ERROR = "VALIDATION_ERROR"
    INVALID_INPUT = "INVALID_INPUT"
    MISSING_REQUIRED_FIELD = "MISSING_REQUIRED_FIELD"
    
    # Errores de base de datos
    DATABASE_ERROR = "DATABASE_ERROR"
    RECORD_NOT_FOUND = "RECORD_NOT_FOUND"
    DUPLICATE_RECORD = "DUPLICATE_RECORD"
    FOREIGN_KEY_VIOLATION = "FOREIGN_KEY_VIOLATION"
    
    # Errores de autenticación y autorización
    UNAUTHORIZED_ACCESS = "UNAUTHORIZED_ACCESS"
    INSUFFICIENT_PERMISSIONS = "INSUFFICIENT_PERMISSIONS"
    INVALID_CREDENTIALS = "INVALID_CREDENTIALS"
    INVALID_TOKEN = "INVALID_TOKEN"
    TOKEN_EXPIRED = "TOKEN_EXPIRED"
    
    # Errores específicos de sesiones
    SESSION_INVALIDATED = "SESSION_INVALIDATED"
    SESSION_EXPIRED = "SESSION_EXPIRED"
    CONCURRENT_SESSION_DETECTED = "CONCURRENT_SESSION_DETECTED"
    
    # Errores de negocio
    BUSINESS_RULE_VIOLATION = "BUSINESS_RULE_VIOLATION"
    
    # Errores del sistema
    INTERNAL_SERVER_ERROR = "INTERNAL_SERVER_ERROR"
    SERVICE_UNAVAILABLE = "SERVICE_UNAVAILABLE"
    TIMEOUT_ERROR = "TIMEOUT_ERROR"

class APIException(Exception):
    """Excepción base para API"""
    
    def __init__(
        self, 
        message: str, 
        error_code: ErrorCode = ErrorCode.INTERNAL_SERVER_ERROR,
        details: Optional[Dict[str, Any]] = None,
        field: Optional[str] = None
    ):
        self.message = message
        self.error_code = error_code
        self.details = details
        self.field = field
        super().__init__(message)

class ValidationException(APIException):
    """Excepción para errores de validación."""
    
    def __init__(self, message: str, field: Optional[str] = None):
        super().__init__(
            message=message,
            error_code=ErrorCode.VALIDATION_ERROR,
            field=field
        )

class RecordNotFoundException(APIException):
    """Excepción para cuando no se encuentra un registro."""
    
    def __init__(self, resource: str = "Registro"):
        super().__init__(
            message=f"{resource} no encontrado",
            error_code=ErrorCode.RECORD_NOT_FOUND
        )

class DatabaseException(APIException):
    """Excepción para errores de base de datos."""
    
    def __init__(self, message: str = "Error en la base de datos"):
        super().__init__(
            message=message,
            error_code=ErrorCode.DATABASE_ERROR
        )

class DuplicateRecordException(APIException):
    """Excepción para registros duplicados."""
    
    def __init__(self, message: str = "El registro ya existe"):
        super().__init__(
            message=message,
            error_code=ErrorCode.DUPLICATE_RECORD
        )

class ForeignKeyViolationException(APIException):
    """Excepción para violaciones de clave foránea."""
    
    def __init__(self, message: str = "Violación de clave foránea"):
        super().__init__(
            message=message,
            error_code=ErrorCode.FOREIGN_KEY_VIOLATION
        )

class BusinessRuleViolationException(APIException):
    """Excepción para violaciones de reglas de negocio."""
    
    def __init__(self, message: str):
        super().__init__(
            message=message,
            error_code=ErrorCode.BUSINESS_RULE_VIOLATION
        )


class InvalidCredentialsException(APIException):
    """Excepción para credenciales inválidas durante el login."""
    
    def __init__(self, message: str = "Credenciales inválidas"):
        super().__init__(
            message=message,
            error_code=ErrorCode.INVALID_CREDENTIALS,
            details={
                "field": "credentials",
                "user_friendly_message": "El usuario o contraseña son incorrectos. Por favor, verifica tus datos e intenta nuevamente."
            }
        )


class InvalidTokenException(APIException):
    """Excepción para tokens inválidos."""
    
    def __init__(self, message: str = "Token inválido"):
        super().__init__(
            message=message,
            error_code=ErrorCode.INVALID_TOKEN
        )


class TokenExpiredException(APIException):
    """Excepción para tokens expirados."""
    
    def __init__(self, message: str = "Token expirado"):
        super().__init__(
            message=message,
            error_code=ErrorCode.TOKEN_EXPIRED
        )


class SessionInvalidatedException(APIException):
    """
    Excepción para cuando una sesión ha sido invalidada.
    
    Esta excepción se lanza cuando un usuario intenta usar un token
    que ya no es válido porque otro login invalidó su sesión.
    """
    
    def __init__(self, message: str = "Tu sesión ha sido invalidada por un nuevo inicio de sesión"):
        super().__init__(
            message=message,
            error_code=ErrorCode.SESSION_INVALIDATED,
            details={
                "reason": "concurrent_login",
                "action_required": "login_again",
                "user_friendly_message": "Alguien más se ha conectado con tus credenciales, por lo que tu sesión ha sido cerrada por seguridad."
            }
        )


class SessionExpiredException(APIException):
    """Excepción para sesiones expiradas."""
    
    def __init__(self, message: str = "Tu sesión ha expirado"):
        super().__init__(
            message=message,
            error_code=ErrorCode.SESSION_EXPIRED,
            details={
                "reason": "session_timeout",
                "action_required": "login_again",
                "user_friendly_message": "Tu sesión ha expirado por inactividad. Por favor, inicia sesión nuevamente."
            }
        )


class ConcurrentSessionDetectedException(APIException):
    """
    Excepción para cuando se detecta una sesión concurrente.
    
    Esta puede usarse opcionalmente para notificar al usuario que
    está intentando iniciar sesión cuando ya tiene una sesión activa.
    """
    
    def __init__(self, message: str = "Ya tienes una sesión activa"):
        super().__init__(
            message=message,
            error_code=ErrorCode.CONCURRENT_SESSION_DETECTED,
            details={
                "reason": "active_session_exists",
                "action_required": "confirm_new_login",
                "user_friendly_message": "Ya tienes una sesión activa. ¿Deseas cerrarla e iniciar una nueva?"
            }
        )



# Este archivo ha sido generado automáticamente por tai-sql
# No modifiques este archivo directamente

from __future__ import annotations
import os
import re
from urllib.parse import (
    unquote,
    urlparse,
    parse_qs
)
from typing import (
    Optional,
    AsyncGenerator
)
from contextlib import asynccontextmanager
from sqlalchemy import URL
from sqlalchemy.ext.asyncio import (
    create_async_engine,
    AsyncEngine,
    AsyncSession,
    async_sessionmaker
)

class AsyncSessionManager:
    """
    Gestor de sesiones asíncronas para SQLAlchemy.
    
    Esta clase centraliza la gestión del ciclo de vida completo de las sesiones
    SQLAlchemy, proporcionando una interfaz consistente y segura para todas
    las operaciones de base de datos.
    
    Características principales:
    - Configuración automática del engine basada en variables de entorno
    - Context managers para gestión automática de transacciones
    - Soporte para sesiones individuales y transacciones compartidas
    - Manejo robusto de errores con rollback automático
    - Pool de conexiones optimizado para aplicaciones web
    
    Métodos principales:
        `get_session`: Context manager para sesiones automáticas
    
    Atributos:
        `engine` (AsyncEngine): Motor SQLAlchemy configurado
        `session_factory` (async_sessionmaker): Factoría de sesiones

    Configuración del Engine:
        - Pool size: 5 conexiones
        - Max overflow: 5 conexiones adicionales
        - Pool timeout: 30 segundos
        - Pool recycle: 3600 segundos
        - Pre-ping True para detectar conexiones perdidas
    
    Ejemplos de uso:
        ```python
        # Para transacciones automáticas
        async with db_api.session_manager.get_session() as session:
            user = await db_api.user.create(name="Juan", session=session)
            await db_api.post.create(title='Hola', content='Mundo', author_id=user.id, session=session)
            # Commit automático
        # Rollback automático en caso de error
        ```
    """
    
    def __init__(self):
        """Inicializa el gestor de sesiones"""
        self._engine: Optional[AsyncEngine] = None
        self._session_factory: Optional[async_sessionmaker] = None
    
    @property
    def async_driver_mapping(self) -> dict:
        """
        Mapeo de drivers síncronos a sus versiones asíncronas.
        
        Returns:
            dict: Mapeo de drivers
        """
        return {
            'postgresql': 'postgresql+asyncpg',
            'mysql': 'mysql+aiomysql',
            'mssql': 'mssql+aioodbc',
            # Agregar más mapeos según sea necesario
        }
    
    @property
    def engine(self) -> AsyncEngine:
        """Acceso al motor de base de datos"""
        if not self._engine:
            # Configuración desde variable de entorno (recomendado)
            connection_string = os.getenv('CHATBOT_DATABASE_URL')
            if not connection_string:
                raise ValueError('Variable de entorno "CHATBOT_DATABASE_URL" no encontrada')
            url = self.from_connection_string(connection_string)
            
            # Configuración del motor
            engine_kwargs = {
                'echo': False,
                'pool_pre_ping': True,
                'pool_recycle': 3600,
                'pool_size': 5,
                'max_overflow': 5,
                'pool_timeout': 30
            }
            
            self._engine = create_async_engine(url, **engine_kwargs)
        return self._engine
    
    @property
    def session_factory(self) -> async_sessionmaker:
        """Configura la factoría de sesiones"""
        if not self._session_factory:
            if not self.engine:
                raise ValueError("Motor de base de datos no configurado")
            self._session_factory = async_sessionmaker(
                bind=self.engine,
                autoflush=False,
                autocommit=False
            )
        return self._session_factory
    
    @asynccontextmanager
    async def get_session(self) -> AsyncGenerator[AsyncSession, None, None]:
        """
        Context manager para obtener una sesión.
            
        Yields:
            AsyncSession: Sesión SQLAlchemy configurada
            
        Example:
            ```python
            async with db.session_manager.get_session() as session:
                await db.user.create(name="Juan", email="juan@triplealpha.in", session=session)
                # Commit automático al salir del context manager
            ```
        """
        session: AsyncSession = self.session_factory()
        
        try:
            if not session.in_transaction():
                await session.begin()
            
            yield session
            
            # Commit si hay una transacción activa
            if session.in_transaction():
                await session.commit()
            
        except Exception as e:
            # Rollback en caso de error
            if session.in_transaction():
                await session.rollback()
            raise e
        finally:
            # Siempre cerrar la sesión
            await session.close()

    def from_connection_string(self, connection_string: str) -> URL:
        """
        Crea una URL desde un string de conexión.
        
        Args:
            connection_string: String de conexión completo
            
        Returns:
            Instancia de URL configurada desde string
            
        Raises:
            ValueError: Si el string de conexión no es válido
        """
        try:
            
            # Mejorar el parsing para manejar caracteres especiales
            connection_string = connection_string.strip()
            
            # Verificar formato básico
            if '://' not in connection_string:
                raise ValueError("String de conexión debe tener formato: driver://user:pass@host:port/db")
            
            # Usar urlparse con manejo mejorado de caracteres especiales
            parse = urlparse(connection_string)
            
            # Validar componentes esenciales
            if not parse.scheme:
                raise ValueError("Driver no especificado en el string de conexión")
            
            if not parse.hostname:
                raise ValueError("Host no especificado en el string de conexión")
            
            # Transformar el driver a su versión asíncrona
            original_driver = parse.scheme
            async_driver = self.async_driver_mapping.get(original_driver, original_driver)
            
            # Si el driver ya incluye un adaptador (ej: postgresql+asyncpg), mantenerlo
            if '+' in original_driver:
                async_driver = original_driver
            
            # Manejar la base de datos (puede estar vacía para algunos casos)
            database = parse.path[1:] if parse.path and len(parse.path) > 1 else None

            # Manejar puerto con valor por defecto según el driver
            port = parse.port
            if port is None:
                # Asignar puertos por defecto según el driver
                default_ports = {
                    'postgresql': 5432,
                    'mysql': 3306,
                    'sqlite': None,
                    'mssql': 1433,
                    'oracle': 1521
                }
                port = default_ports.get(parse.scheme, None)
            
            # Parsear query parameters de forma más robusta
            query_params = {}
            if parse.query:
                try:
                    query_params = parse_qs(parse.query)
                    # Convertir listas de un elemento a valores únicos
                    query_params = {k: v[0] if len(v) == 1 else v for k, v in query_params.items()}
                except Exception as e:
                    # Si falla el parsing de query, continuar sin ellos
                    print(f"⚠️  Advertencia: No se pudieron parsear los parámetros de consulta: {e}")
                    query_params = {}
            
            # Crear la URL con manejo de errores mejorado
            try:
                return URL.create(
                    drivername=async_driver,
                    username=parse.username,
                    password=parse.password,  # urlparse maneja el unquoting automáticamente
                    host=parse.hostname,
                    port=port,
                    database=database,
                    query=query_params
                )
            except Exception as url_error:
                # Proporcionar información más detallada del error
                raise ValueError(
                    f"Error creando URL SQLAlchemy: {url_error}\n"
                    f"Componentes parseados:\n"
                    f"  - Driver: {parse.scheme}\n"
                    f"  - Usuario: {parse.username}\n"
                    f"  - Host: {parse.hostname}\n"
                    f"  - Puerto: {port}\n"
                    f"  - Base de datos: {database}\n"
                    f"  - Tiene contraseña: {'Sí' if parse.password else 'No'}"
                )
            
        except Exception as e:
            return self.from_connection_string_escaped(connection_string)

    def from_connection_string_escaped(self, connection_string: str) -> URL:
        """
        Versión alternativa que maneja manualmente el escaping de caracteres especiales.
        
        Útil cuando urlparse falla con caracteres especiales en las contraseñas.
        
        Args:
            connection_string: String de conexión que puede contener caracteres especiales
            
        Returns:
            URL
        """
        
        try:
            
            # Parsear manualmente para manejar caracteres especiales
            # Patrón para extraer componentes: driver://user:pass@host:port/db
            pattern = r'^([^:]+)://([^:]+):([^@]+)@([^:/]+)(?::(\d+))?(?:/(.*))?$'
            match = re.match(pattern, connection_string.strip())
            
            if not match:
                raise ValueError(
                    "Formato de connection string no válido.\n"
                    "Esperado: driver://username:password@host:port/database"
                )
            
            driver, username, password, host, port, database_and_query = match.groups()

            async_driver = self.async_driver_mapping.get(driver, driver)
            
            # Separar database de query parameters si existen
            database = None
            query_params = {}
            
            if database_and_query:
                if '?' in database_and_query:
                    database, query_string = database_and_query.split('?', 1)
                    # Parsear query parameters
                    for param in query_string.split('&'):
                        if '=' in param:
                            key, value = param.split('=', 1)
                            query_params[unquote(key)] = unquote(value)
                else:
                    database = database_and_query
            
            # Convertir puerto a entero si existe
            if port:
                try:
                    port = int(port)
                except ValueError:
                    raise ValueError(f"Puerto inválido: {port}")
            
            else:
                # Asignar puertos por defecto según el driver
                default_ports = {
                    'postgresql': 5432,
                    'mysql': 3306,
                    'sqlite': None,
                    'mssql': 1433,
                    'oracle': 1521
                }
                port = default_ports.get(driver, None)
            
            # Crear URL SQLAlchemy
            return URL.create(
                drivername=async_driver,
                username=unquote(username) if username else None,
                password=unquote(password) if password else None,
                host=unquote(host) if host else None,
                port=port,
                database=unquote(database) if database else None,
                query=query_params
            )
            
        except Exception as e:
            raise ValueError(f"Error en parsing manual: {e}")



from fastapi import APIRouter, Depends, Query, Path, Body
from typing import Optional, List, Literal
from tai_alphi import Alphi
from datetime import datetime

from ..database import *
from ..resources import (
    APIResponse, PaginatedResponse, RecordNotFoundException
)


logger = Alphi.get_logger_by_name("tai-chatbot")

usuario_router = APIRouter(
    prefix="/usuario",
    tags=["Usuario"]
)

@usuario_router.get("", 
    response_model=APIResponse[List[UsuarioRead]],
    response_description="Lista de registros de usuario obtenido exitosamente",
    operation_id="usuario_find_many",
    summary="Busca varios registros en la tabla usuario",
    responses={
        200: {
            "description": "Lista de registros de usuario obtenido exitosamente",
            "content": {
                "application/json": {
                    "schema": {
                        "$ref": "#/components/schemas/APIResponse_List_UsuarioRead__"
                    }
                }
            },
            "links": {
                "self": {
                    "operationId": "usuario_find_many",
                    "description": "Enlace a la consulta actual con los mismos filtros",
                    "parameters": {
                        "username": "$request.query.username",
                        "email": "$request.query.email",
                        "avatar": "$request.query.avatar",
                        "session_id": "$request.query.session_id",
                        "created_at": "$request.query.created_at",
                        "updated_at": "$request.query.updated_at",
                        "is_active": "$request.query.is_active",
                        "limit": "$request.query.limit",
                        "offset": "$request.query.offset",
                        "order_by": "$request.query.order_by",
                        "order": "$request.query.order",
                        "includes": "$request.query.includes"
                    }
                },
                "item": {
                    "operationId": "usuario_find",
                    "description": "Enlace para acceder a un elemento específico",
                    "parameters": {
                        "username": "$response.body#/data/**/username",
                        "includes": "$request.query.includes"
                    }
                },
                "create": {
                    "operationId": "usuario_create",
                    "description": "Enlace para crear un nuevo Usuario"
                },
                "count": {
                    "operationId": "usuario_count",
                    "description": "Enlace para obtener el conteo total con los mismos filtros",
                    "parameters": {
                        "username": "$request.query.username",
                        "email": "$request.query.email",
                        "avatar": "$request.query.avatar",
                        "session_id": "$request.query.session_id",
                        "created_at": "$request.query.created_at",
                        "updated_at": "$request.query.updated_at",
                        "is_active": "$request.query.is_active",
                    }
                },
            "chats": {
            "operationId": "chat_find_many",
                    "description": "Enlace a los Chats relacionados",
                    "parameters": {
                        "username": "$response.body#/data/**/username",
                        "includes": "$request.query.includes"
                    }
            }
            }
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "El límite no puede ser negativo",
                                "field": "limit",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_find_many(
    limit: Optional[int] = Query(None, description="Número de registros a retornar. Valores positivos toman los n primeros registros, valores negativos toman los n últimos registros (requiere order_by)", gt=0),
    order_by: List[str] = Query(None, description="Lista de nombres de columnas para ordenar los resultados.⚠️ **IMPORTANTE**: los nombres de columnas deben existir, si no serán omitidas. Requerido cuando limit es negativo"),
    order: Optional[Literal["ASC", "DESC"]] = Query("ASC", description="Dirección de ordenamiento: 'ASC' para ascendente (por defecto), 'DESC' para descendente. Solo aplica si order_by está definido", regex="^(ASC|DESC)$"),
    offset: Optional[int] = Query(None, description="Número de registros a omitir desde el inicio. Útil para paginación. Debe ser un valor no negativo", ge=0),
    username: Optional[str] = Query(None, description="Filtrar por username (clave primaria)"),
    email: Optional[str] = Query(None, description='Filtrar por email. ⚠️ **IMPORTANTE**: utiliza "%email%" para hacer consultas ILIKE. Correo electrónico del usuario', min_length=1, max_length=255),
    avatar: Optional[str] = Query(None, description='Filtrar por avatar. ⚠️ **IMPORTANTE**: utiliza "%avatar%" para hacer consultas ILIKE. URL del avatar del usuario', min_length=1, max_length=255),
    session_id: Optional[str] = Query(None, description='Filtrar por session_id. ⚠️ **IMPORTANTE**: utiliza "%session_id%" para hacer consultas ILIKE. ID de la sesión activa del usuario', min_length=1, max_length=255),
    min_created_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_created_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    min_updated_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para updated_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_updated_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para updated_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    is_active: Optional[bool] = Query(None, description="Filtrar por is_active (verdadero/falso). Estado activo del usuario"),
    includes: List[str] = Query(None, description="Lista de relaciones a incluir en la respuesta para obtener datos relacionados. Especifica los nombres de las relaciones que deseas expandir"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[List[UsuarioRead]]:
    """
    ## Resumen
    Obtiene una lista de `usuarios` con filtros opcionales y soporte para paginación.
    
    Este endpoint permite realizar búsquedas flexibles aplicando filtros opcionales
    por cualquiera de los campos disponibles, con soporte completo para paginación
    mediante los parámetros limit y offset.

    ## Resultado
    En `APIResponse.data`, retorna un listado de objetos donde cada uno representa un registro de la tabla `usuario` que incluye todos sus atributos

    ## Datos
    Para cada registro en `data` se incluye:
    - **username** (str): Nombre de usuario único
    - **password** (str): Contraseña encriptada
    - **email** (str, opcional): Correo electrónico del usuario
    - **avatar** (str, opcional): URL del avatar del usuario
    - **session_id** (str, opcional): ID de la sesión activa del usuario
    - **created_at** (datetime): Fecha de creación del usuario
    - **updated_at** (datetime): Fecha de última actualización
    - **is_active** (bool): Estado activo del usuario
    
    ## Parámetros de Filtrado
    
    Todos los parámetros de filtrado son opcionales y se pueden combinar:
    - **username**: Filtrar por username
    - **email**: Filtrar por email. ⚠️ **IMPORTANTE**: utiliza "%email%" para hacer consultas ILIKE.
    - **avatar**: Filtrar por avatar. ⚠️ **IMPORTANTE**: utiliza "%avatar%" para hacer consultas ILIKE.
    - **session_id**: Filtrar por session_id. ⚠️ **IMPORTANTE**: utiliza "%session_id%" para hacer consultas ILIKE.
    - **min_created_at**: Filtrar por valor mínimo de created_at (incluído el valor del filtro)
    - **max_created_at**: Filtrar por valor máximo de created_at (incluído el valor del filtro)
    - **min_updated_at**: Filtrar por valor mínimo de updated_at (incluído el valor del filtro)
    - **max_updated_at**: Filtrar por valor máximo de updated_at (incluído el valor del filtro)
    - **is_active**: Filtrar por is_active (verdadero/falso)

    
    ## Parámetros de Paginación
    
    - **limit**: Número máximo de registros a retornar. Solo admite valores positivos. Si no se especifica, retorna todos los registros que coincidan con los filtros.
    - **order_by**: Lista de nombres de columnas para ordenar los resultados.⚠️ **IMPORTANTE**: los nombres de columnas deben existir, si no serán omitidas.
    - **order**: Dirección de ordenamiento: 'ASC' para ascendente (por defecto), 'DESC' para descendente. Solo aplica si order_by está definido.
    - **offset**: Número de registros a omitir desde el inicio. Solo admite valores positivos. Si no se especifica, inicia desde el primer registro.
    
    ## Consulta combinada (recomendado para pocos registros)
    ⚠️ **IMPORTANTE**: Usa siempre el parámetro `includes` para cargar relaciones en una sola consulta y evitar múltiples llamadas al API.
    
    ⚠️ **WARNING**: Si la relación incluida tiene muchos registros relacionados, la respuesta puede ser muy grande y lenta. Mejor consultar su endpoint directamente con filtros.
    
    El parametro `includes` permite cargar relaciones asociadas a los registros.

    ### Relaciones disponibles
    - **chats**: lista de Chat relacionados (one-to-many)

        - **descripción**: Tabla que almacena las conversaciones del chatbot
    
    ### Ejemplos básicos:
    #### Solo datos básicos
    `usuario = GET /usuario`
    
    #### Incluir chats
    `usuario = GET /usuario?includes=chats`
    
    #### Relaciones anidadas
    Puedes incluir los datos de chats y además incluir sus propias relaciones  
    `usuario = GET /usuario?includes=chats.{nested_relation}`  
    """
    result = await api.usuario.find_many(
        limit=limit,
        offset=offset,
        order_by=order_by,
        order=order,
        username=username,
        email=email,
        avatar=avatar,
        session_id=session_id,
        min_created_at=min_created_at,
        max_created_at=max_created_at,
        min_updated_at=min_updated_at,
        max_updated_at=max_updated_at,
        is_active=is_active,
        includes=includes
    )
    
    # Obtener el total para metadatos de paginación si es necesario
    total = None
    if limit is not None or offset is not None:
        try:
            total = await api.usuario.count(
                username=username,
                email=email,
                avatar=avatar,
                session_id=session_id,
                min_created_at=min_created_at,
                max_created_at=max_created_at,
                min_updated_at=min_updated_at,
                max_updated_at=max_updated_at,
                is_active=is_active,
            )
        except Exception as e:
            logger.warning(f"No se pudo obtener el total de registros: {str(e)}")
    
    return PaginatedResponse.success_paginated(
        data=result,
        total=total,
        limit=limit,
        offset=offset,
        message=f"Usuarios obtenidos exitosamente"
    )

@usuario_router.get("/{username:str}", 
    response_model=APIResponse[UsuarioRead],
    response_description="Registro único de usuario obtenido exitosamente",
    operation_id="usuario_find",
    summary="Busca un registro en la tabla usuario",
    responses={
        200: {
            "description": "Registro único de usuario obtenido exitosamente",
            "content": {
                "application/json": {
                    "schema": {
                        "$ref": "#/components/schemas/APIResponse_UsuarioRead_"
                    }
                }
            },
            "links": {
                "self": {
                    "operationId": "usuario_find",
                    "description": "Enlace al recurso actual",
                    "parameters": {
                        "username": "$response.body#/data/username",
                        "includes": "$request.query.includes"
                    }
                },
                "collection": {
                    "operationId": "usuario_find_many",
                    "description": "Enlace a la colección de Usuarios"
                },
                "edit": {
                    "operationId": "usuario_update",
                    "description": "Enlace para actualizar este Usuario",
                    "parameters": {
                        "username": "$response.body#/data/username",
                    }
                },
                "delete": {
                    "operationId": "usuario_delete",
                    "description": "Enlace para eliminar este Usuario",
                    "parameters": {
                        "username": "$response.body#/data/username",
                    }
                },
            "chats": {
            "operationId": "chat_find_many",
                    "description": "Enlace a los Chats relacionados",
                    "parameters": {
                        "username": "$response.body#/data/username",
                        "includes": "$request.query.includes"
                    }
            }
            }
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "username debe ser mayor a 0",
                                "field": "username",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        404: {
            "description": "Usuario no encontrado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Usuario no encontrado",
                        "errors": [
                            {
                                "code": "RECORD_NOT_FOUND",
                                "message": "Usuario no encontrado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_find(
    username: str = Path(..., description="Nombre de usuario único"),
    includes: List[str] = Query(None, description="Lista de relaciones a incluir en la respuesta para obtener datos relacionados. Especifica los nombres de las relaciones que deseas expandir"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[UsuarioRead]:
    """
    ## Resumen
    Obtiene un Usuario específico por su clave primaria.
    
    Este endpoint permite recuperar un registro individual de Usuario
    utilizando su identificador único (clave primaria). Opcionalmente puede
    incluir datos de relaciones asociadas.

    ## Resultado
    Si la consulta es exitosa, en `APIResponse.data`, retorna un objeto que representa un registro de la tabla `usuario` que incluye todos sus atributos

    Si no se encuentra el registro, devuelve un error 404 `RECORD_NOT_FOUND`.

    ## Datos
    Para cada registro en `data` se incluye:
    - **username** (str): Nombre de usuario único
    - **password** (str): Contraseña encriptada
    - **email** (str, opcional): Correo electrónico del usuario
    - **avatar** (str, opcional): URL del avatar del usuario
    - **session_id** (str, opcional): ID de la sesión activa del usuario
    - **created_at** (datetime): Fecha de creación del usuario
    - **updated_at** (datetime): Fecha de última actualización
    - **is_active** (bool): Estado activo del usuario
    
    ## Parámetros de Identificación
    
    - **username**: username del Usuario a buscar (tipo: str)
    
    ## Consulta combinada (RECOMENDADO)
    ⚠️ **IMPORTANTE**: Usa siempre el parámetro `includes` para cargar relaciones en una sola consulta y evitar múltiples llamadas al API.
    
    El parametro `includes` permite cargar relaciones asociadas a los registros.

    ### Relaciones disponibles (usar con parámetro 'includes'):
    - chats: Lista de Chat relacionados (one-to-many)
        Tabla que almacena las conversaciones del chatbot
    
    ### Uso del parámetro 'includes':
    Para cargar relaciones específicas, usa el parámetro 'includes' en la consulta:
    
    ### Ejemplos básicos:
    #### Solo datos básicos
    `usuario = GET /usuario/{username:str}`
    
    #### Incluir chats
    `usuario = GET /usuario/{username:str}?includes=chats`
    
    #### Relaciones anidadas
    Puedes incluir los datos de chats y además incluir sus propias relaciones  
    `usuario = GET /usuario/{username:str}?includes=chats.{nested_relation}`
    """
    # Validaciones básicas de entrada
    
    result = await api.usuario.find(
        username=username,
        includes=includes
    )
    
    if result is None:
        raise RecordNotFoundException("Usuario")
        
    return APIResponse.success(
        data=result,
        message="Usuario obtenido exitosamente"
    )

@usuario_router.get("/count", 
    response_model=APIResponse[int],
    response_description="Número de registros de usuario según los filtros aplicados",
    operation_id="usuario_count",
    summary="Cuenta registros en la tabla usuario",
    responses={
        200: {
            "description": "Conteo realizado exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": 42,
                        "message": "Conteo realizado exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_count(
    username: Optional[str] = Query(None, description="Filtrar por username (clave primaria)"),
    email: Optional[str] = Query(None, description='Filtrar por email. ⚠️ **IMPORTANTE**: utiliza "%email%" para hacer consultas ILIKE. Correo electrónico del usuario', min_length=1, max_length=255),
    avatar: Optional[str] = Query(None, description='Filtrar por avatar. ⚠️ **IMPORTANTE**: utiliza "%avatar%" para hacer consultas ILIKE. URL del avatar del usuario', min_length=1, max_length=255),
    session_id: Optional[str] = Query(None, description='Filtrar por session_id. ⚠️ **IMPORTANTE**: utiliza "%session_id%" para hacer consultas ILIKE. ID de la sesión activa del usuario', min_length=1, max_length=255),
    min_created_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_created_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    min_updated_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para updated_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_updated_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para updated_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    is_active: Optional[bool] = Query(None, description="Filtrar por is_active (verdadero/falso). Estado activo del usuario"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Cuenta el número de Usuarios que coinciden con los filtros.
    """
    result = await api.usuario.count(
        username=username,
        email=email,
        avatar=avatar,
        session_id=session_id,
        min_created_at=min_created_at,
        max_created_at=max_created_at,
        min_updated_at=min_updated_at,
        max_updated_at=max_updated_at,
        is_active=is_active,
    )
    
    return APIResponse.success(
        data=result,
        message="Conteo realizado exitosamente"
    )

@usuario_router.get("/exists", 
    response_model=APIResponse[bool],
    operation_id="usuario_exists",
    summary="Verifica existencia en la tabla usuario",
    responses={
        200: {
            "description": "Verificación realizada exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": True,
                        "message": "Verificación realizada exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_exists(
    username: Optional[str] = Query(None, description="Filtrar por username (clave primaria)"),
    email: Optional[str] = Query(None, description='Filtrar por email. ⚠️ **IMPORTANTE**: utiliza "%email%" para hacer consultas ILIKE. Correo electrónico del usuario', min_length=1, max_length=255),
    avatar: Optional[str] = Query(None, description='Filtrar por avatar. ⚠️ **IMPORTANTE**: utiliza "%avatar%" para hacer consultas ILIKE. URL del avatar del usuario', min_length=1, max_length=255),
    session_id: Optional[str] = Query(None, description='Filtrar por session_id. ⚠️ **IMPORTANTE**: utiliza "%session_id%" para hacer consultas ILIKE. ID de la sesión activa del usuario', min_length=1, max_length=255),
    min_created_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_created_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para created_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    min_updated_at: Optional[datetime] = Query(None, description="Fecha y hora mínima para updated_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    max_updated_at: Optional[datetime] = Query(None, description="Fecha y hora máxima para updated_at (incluida, formato: YYYY-MM-DDTHH:MM:SS)"),
    is_active: Optional[bool] = Query(None, description="Filtrar por is_active (verdadero/falso). Estado activo del usuario"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[bool]:
    """
    Verifica si existe al menos un usuario que coincida con los filtros.
    """
    result = await api.usuario.exists(
        username=username,
        email=email,
        avatar=avatar,
        session_id=session_id,
        min_created_at=min_created_at,
        max_created_at=max_created_at,
        min_updated_at=min_updated_at,
        max_updated_at=max_updated_at,
        is_active=is_active,
    )
    
    return APIResponse.success(
        data=result,
        message="Verificación realizada exitosamente"
    )

@usuario_router.post("", 
    response_model=APIResponse[UsuarioRead],
    status_code=201,
    operation_id="usuario_create",
    summary="Crea un registro en la tabla usuario",
    responses={
        422: {
            "description": "Error de validación en los datos de entrada",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "El campo es requerido",
                                "field": "password",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        409: {
            "description": "Registro duplicado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "El registro ya existe",
                        "errors": [
                            {
                                "code": "DUPLICATE_RECORD",
                                "message": "El registro ya existe",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Violación de clave foránea",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Violación de clave foránea",
                        "errors": [
                            {
                                "code": "FOREIGN_KEY_VIOLATION",
                                "message": "La referencia especificada no existe",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_create(
    usuario: UsuarioCreate,
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[UsuarioRead]:
    """
    Crea un nuevo Usuario.
    """
    result = await api.usuario.create(usuario)
    
    return APIResponse.success(
        data=result,
        message="Usuario creado exitosamente"
    )

@usuario_router.patch("/{username:str}", 
    response_model=APIResponse[int],
    operation_id="usuario_update",
    summary="Actualiza un registro en la tabla usuario",
    responses={
        200: {
            "description": "Usuario actualizado exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": 1,
                        "message": "Usuario actualizado exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Error de validación en parámetros o datos",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "username debe ser mayor a 0",
                                "field": "username",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        404: {
            "description": "Usuario no encontrado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Usuario no encontrado",
                        "errors": [
                            {
                                "code": "RECORD_NOT_FOUND",
                                "message": "Usuario no encontrado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Violación de clave foránea",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Violación de clave foránea",
                        "errors": [
                            {
                                "code": "FOREIGN_KEY_VIOLATION",
                                "message": "La referencia especificada no existe",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_update(
    username: str = Path(..., description="Nombre de usuario único"),
    values: UsuarioUpdateValues = Body(...),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Actualiza un Usuario específico.
    """
    # Validaciones básicas de entrada
    
    # Verificar que el registro existe antes de actualizar
    existing = await api.usuario.find(
        username=username,
    )
    
    if existing is None:
        raise RecordNotFoundException("Usuario")
    
    result = await api.usuario.update(
        username=username,
        updated_values=values
    )
    
    if result == 0:
        raise RecordNotFoundException("Usuario")
        
    return APIResponse.success(
        data=result,
        message="Usuario actualizado exitosamente"
    )

@usuario_router.patch("", 
    response_model=APIResponse[int],
    operation_id="usuario_update_many",
    summary="Actualiza múltiples registros en la tabla usuario",
    responses={
        200: {
            "description": "Usuarios actualizados exitosamente",
            "content": {
                "application/json": {
                    "examples": {
                        "records_updated": {
                            "summary": "Registros actualizados",
                            "value": {
                                "status": "success",
                                "data": 5,
                                "message": "5 Usuarios actualizados exitosamente",
                                "errors": None,
                                "meta": None
                            }
                        },
                        "no_records_found": {
                            "summary": "No se encontraron registros",
                            "value": {
                                "status": "success",
                                "data": 0,
                                "message": "No se encontraron registros que coincidan con los criterios",
                                "errors": None,
                                "meta": None
                            }
                        }
                    }
                }
            }
        },
        422: {
            "description": "Error de validación en los datos",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "Los criterios de búsqueda son requeridos",
                                "field": "filters",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_update_many(
    payload: UsuarioUpdate,
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Actualiza múltiples Usuarios.
    """
    result = await api.usuario.update_many(payload)
    
    message = f"{result} Usuarios actualizados exitosamente" if result > 0 else "No se encontraron registros que coincidan con los criterios"
    
    return APIResponse.success(
        data=result,
        message=message
    )

@usuario_router.delete("/{username:str}", 
    response_model=APIResponse[int],
    operation_id="usuario_delete",
    summary="Elimina un registro en la tabla usuario",
    responses={
        200: {
            "description": "Usuario eliminado exitosamente",
            "content": {
                "application/json": {
                    "example": {
                        "status": "success",
                        "data": 1,
                        "message": "Usuario eliminado exitosamente",
                        "errors": None,
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Error de validación en parámetros",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error de validación",
                        "errors": [
                            {
                                "code": "VALIDATION_ERROR",
                                "message": "username debe ser mayor a 0",
                                "field": "username",
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        404: {
            "description": "Usuario no encontrado",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Usuario no encontrado",
                        "errors": [
                            {
                                "code": "RECORD_NOT_FOUND",
                                "message": "Usuario no encontrado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        422: {
            "description": "Violación de clave foránea",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Violación de clave foránea",
                        "errors": [
                            {
                                "code": "FOREIGN_KEY_VIOLATION",
                                "message": "No se puede eliminar el registro porque está siendo referenciado",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        },
        500: {
            "description": "Error interno del servidor",
            "content": {
                "application/json": {
                    "example": {
                        "status": "error",
                        "data": None,
                        "message": "Error interno del servidor",
                        "errors": [
                            {
                                "code": "DATABASE_ERROR",
                                "message": "Error en la base de datos",
                                "field": None,
                                "details": None
                            }
                        ],
                        "meta": None
                    }
                }
            }
        }
    }
)
async def usuario_delete(
    username: str = Path(..., description="Nombre de usuario único"),
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[int]:
    """
    Elimina un Usuario por su primary key.
    """
    # Validaciones básicas de entrada
    
    # Verificar que el registro existe antes de eliminar
    existing = await api.usuario.find(
        username=username,
    )
    
    if existing is None:
        raise RecordNotFoundException("Usuario")
    
    result = await api.usuario.delete(
        username=username,
    )
    
    if result == 0:
        raise RecordNotFoundException("Usuario")
        
    return APIResponse.success(
        data=result,
        message="Usuario eliminado exitosamente"
    )



"""
Gestor de conexiones WebSocket para chat streaming
"""
from __future__ import annotations
from typing import Dict, Set, Optional
from pydantic import BaseModel
from fastapi import WebSocket
import json
from datetime import datetime
import asyncio
import uuid
from tai_alphi import Alphi

from .resources.exceptions import APIException
from .config import ConnectionConfig

logger = Alphi.get_logger_by_name("tai-chatbot")

class Connection(BaseModel):
    """Modelo de conexión WebSocket"""
    websocket: WebSocket
    username: str
    chat_id: int
    connected_at: datetime
    
    class Config:
        arbitrary_types_allowed = True  # Permite tipos como WebSocket

class ConnectionManager:
    """Gestor de conexiones WebSocket para chat streaming"""
    
    def __init__(self, config: Optional[ConnectionConfig] = None):
        # Cargar configuración
        self.config = config or ConnectionConfig.load()
        
        # connection_id -> Connection
        self.connections: Dict[str, Connection] = {}
        # username -> Set[connection_id] 
        self.user_connections: Dict[str, Set[str]] = {}
        # chat_id -> Set[connection_id]
        self.chat_connections: Dict[int, Set[str]] = {}
        # connection_id -> last_activity
        self.last_activity: Dict[str, datetime] = {}
        
        # Iniciar tarea de limpieza
        self._cleanup_task = None
    
    def _check_connection_limit(self) -> bool:
        """Verificar si se puede aceptar una nueva conexión"""
        return len(self.connections) < self.config.max_concurrent_connections
    
    async def connect(
        self, 
        websocket: WebSocket, 
        username: str,
        chat_id: int
    ) -> str:
        """
        Conectar un nuevo WebSocket
        
        Args:
            websocket: Conexión WebSocket
            username: ID del usuario
            chat_id: ID del chat
            
        Returns:
            str: ID de conexión único
            
        Raises:
            Exception: Si se alcanza el límite de conexiones
        """
        # Verificar límite de conexiones
        if not self._check_connection_limit():
            await websocket.close(code=1008, reason="Límite de conexiones alcanzado")
            raise APIException(
                f"Límite de conexiones alcanzado",
                details={'max_concurrent_connections': self.config.max_concurrent_connections}
            )
        
        await websocket.accept()
        
        connection_id = str(uuid.uuid4())
        now = datetime.now()
        
        # Registrar conexión
        self.connections[connection_id] = Connection(
            websocket=websocket,
            username=username,
            chat_id=chat_id,
            connected_at=now
        )
        
        # Registrar por usuario
        if username not in self.user_connections:
            self.user_connections[username] = set()
        self.user_connections[username].add(connection_id)
        
        # Registrar por chat
        if chat_id not in self.chat_connections:
            self.chat_connections[chat_id] = set()
        self.chat_connections[chat_id].add(connection_id)
        
        # Actualizar actividad
        self.last_activity[connection_id] = now
        
        logger.info(f"Nueva conexión WebSocket: {connection_id} (user: {username}, chat: {chat_id})")
        
        # Iniciar tarea de limpieza si no existe
        if self._cleanup_task is None or self._cleanup_task.done():
            self._cleanup_task = asyncio.create_task(self._periodic_cleanup())
            pass
        
        return connection_id
    
    async def disconnect(self, connection_id: str):
        """
        Desconectar un WebSocket
        
        Args:
            connection_id: ID de la conexión
        """
        if connection_id not in self.connections:
            return
        
        connection = self.connections[connection_id]
        username = connection.username
        chat_id = connection.chat_id
        
        # Remover de registros
        del self.connections[connection_id]
        
        if connection_id in self.last_activity:
            del self.last_activity[connection_id]
        
        # Remover de usuario
        if username in self.user_connections:
            self.user_connections[username].discard(connection_id)
            if not self.user_connections[username]:
                del self.user_connections[username]
        
        # Remover de chat
        if chat_id in self.chat_connections:
            self.chat_connections[chat_id].discard(connection_id)
            if not self.chat_connections[chat_id]:
                del self.chat_connections[chat_id]
        
        logger.info(f"Conexión WebSocket desconectada: {connection_id}")
    
    async def send_to_connection(
        self, 
        connection_id: str, 
        data: Dict
    ) -> bool:
        """
        Enviar datos a una conexión específica
        
        Args:
            connection_id: ID de la conexión
            data: Datos a enviar
            
        Returns:
            bool: True si se envió correctamente
        """
        if connection_id not in self.connections:
            return False
        
        try:
            websocket = self.connections[connection_id].websocket
            await websocket.send_text(json.dumps(data))
            self.last_activity[connection_id] = datetime.now()
            return True
        except Exception as e:
            logger.error(f"Error enviando a conexión {connection_id}: {e}")
            await self.disconnect(connection_id)
            return False
    
    async def send_to_chat(self, chat_id: str, data: Dict) -> int:
        """
        Enviar datos a todas las conexiones de un chat
        
        Args:
            chat_id: ID del chat
            data: Datos a enviar
            
        Returns:
            int: Número de conexiones que recibieron el mensaje
        """
        if chat_id not in self.chat_connections:
            return 0
        
        sent_count = 0
        connections_to_remove = []
        
        for connection_id in self.chat_connections[chat_id].copy():
            success = await self.send_to_connection(connection_id, data)
            if success:
                sent_count += 1
            else:
                connections_to_remove.append(connection_id)
        
        # Limpiar conexiones fallidas
        for connection_id in connections_to_remove:
            await self.disconnect(connection_id)
        
        return sent_count
    
    async def send_to_user(self, username: str, data: Dict) -> int:
        """
        Enviar datos a todas las conexiones de un usuario
        
        Args:
            username: ID del usuario
            data: Datos a enviar
            
        Returns:
            int: Número de conexiones que recibieron el mensaje
        """
        if username not in self.user_connections:
            return 0
        
        sent_count = 0
        connections_to_remove = []
        
        for connection_id in self.user_connections[username].copy():
            success = await self.send_to_connection(connection_id, data)
            if success:
                sent_count += 1
            else:
                connections_to_remove.append(connection_id)
        
        # Limpiar conexiones fallidas
        for connection_id in connections_to_remove:
            await self.disconnect(connection_id)
        
        return sent_count
    
    def get_connection_info(self, connection_id: str) -> Optional[Connection]:
        """Obtener información de una conexión"""
        return self.connections.get(connection_id)
    
    def get_active_connections_count(self) -> int:
        """Obtener número total de conexiones activas"""
        return len(self.connections)
    
    def get_chat_connections_count(self, chat_id: str) -> int:
        """Obtener número de conexiones activas para un chat"""
        return len(self.chat_connections.get(chat_id, set()))
    
    def get_user_connections_count(self, username: str) -> int:
        """Obtener número de conexiones activas para un usuario"""
        return len(self.user_connections.get(username, set()))
    
    def get_config(self) -> ConnectionConfig:
        """Obtener la configuración actual"""
        return self.config
    
    def update_config(self, new_config: ConnectionConfig) -> None:
        """Actualizar la configuración"""
        self.config = new_config
        logger.info(f"Configuración WebSocket actualizada: timeout={self.config.websocket_timeout}s, max_connections={self.config.max_concurrent_connections}")
    
    def reload_config(self) -> None:
        """Recargar configuración desde .chatbotconfig"""
        new_config = ConnectionConfig.load()
        self.update_config(new_config)
    
    async def _periodic_cleanup(self):
        """Tarea periódica de limpieza de conexiones inactivas"""
        while True:
            try:
                await asyncio.sleep(self.config.cleanup_interval)
                await self._cleanup_stale_connections()
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"Error en limpieza periódica: {e}")
    
    async def _cleanup_stale_connections(self):
        """Limpiar conexiones inactivas o cerradas"""
        now = datetime.now()
        timeout_seconds = self.config.websocket_timeout
        
        stale_connections = []
        
        for connection_id, last_activity in self.last_activity.items():
            # Verificar timeout
            if (now - last_activity).total_seconds() > timeout_seconds:
                stale_connections.append(connection_id)
                continue
            
            # Verificar si la conexión está cerrada
            if connection_id in self.connections:
                websocket = self.connections[connection_id].websocket
                try:
                    # Enviar ping para verificar conexión
                    await websocket.send_text(json.dumps({"type": "ping"}))
                except Exception:
                    stale_connections.append(connection_id)
        
        # Limpiar conexiones stale
        for connection_id in stale_connections:
            await self.disconnect(connection_id)
        
        if stale_connections:
            logger.info(f"Limpiadas {len(stale_connections)} conexiones inactivas")

# Cargar configuración global
websocket_config = ConnectionConfig.load()

# Instancia global del gestor de conexiones con configuración
connection_manager = ConnectionManager(config=websocket_config)



"""
Router de chat streaming con WebSocket
"""
from datetime import datetime
import json
import asyncio
from tai_alphi import Alphi
from langchain_core.messages import HumanMessage
from fastapi import APIRouter, WebSocket, WebSocketDisconnect, Depends, HTTPException, Query

from api.auth import get_current_user, JWTHandler
from api.resources import (
    RecordNotFoundException,
    APIException,
    APIResponse
)
from ..database import chatbot_api, UsuarioRead

from ..utils import add_message
from ..types import (
    StreamMessageRequest,
    StreamMessageResponse,
    StreamingStatus,
    WebsocketMessageType
)
from ..llm import llm_manager
from ..websockets import connection_manager

logger = Alphi.get_logger_by_name("tai-api")

# Router
streaming_router = APIRouter(
    prefix="/streaming",
    tags=["Streaming"]
)
        
@streaming_router.post("/chat/send", response_model=APIResponse[StreamMessageResponse])
async def send_message_to_stream(
    input_msg: StreamMessageRequest,
    current_user: UsuarioRead = Depends(get_current_user)
) -> APIResponse[StreamMessageResponse]:
    """
    Enviar mensaje y iniciar streaming de respuesta
    
    Este endpoint:
    1. Valida el chat y permisos
    2. Guarda el mensaje del usuario
    3. Notifica via WebSocket que el streaming comenzó
    4. Retorna inmediatamente mientras el streaming continúa en background
    """
    try:
        
        # 1. Validar que el chat existe y pertenece al usuario
        chat = await chatbot_api.chat.find(id=input_msg.chat_id)
        if not chat:
            raise RecordNotFoundException(resource="Chat")
        
        if chat.username != current_user.username:
            raise APIException(message="No tienes permisos para este chat")
        
        msg = HumanMessage(content=input_msg.content)
        
        response = await add_message(chat_id=chat.id, msg=msg)

        if not response.get("success"):
            raise APIException(
                message="Error guardando el mensaje del usuario",
                details={"error": response.get("error")}
            )
        
        # 3. Notificar inicio de streaming via WebSocket
        connections_notified = await connection_manager.send_to_chat(
            chat.id, {"type": WebsocketMessageType.MESSAGE_RECEIVED.value}
        )
        
        # 4. Iniciar streaming en background
        asyncio.create_task(llm_manager.streaming_response(msg, chat.id))
        
        response_data = StreamMessageResponse(
            user_id=current_user.username,
            msg_id=response.get('msg').id,
            connections_notified=connections_notified,
            chat_id=chat.id
        )
        
        return APIResponse.success(
            data=response_data,
            message="Mensaje enviado, streaming iniciado"
        )
        
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error en send_message_to_stream: {e}")
        raise HTTPException(status_code=500, detail=str(e))

@streaming_router.websocket("/ws/chat/{chat_id}")
async def websocket_chat_endpoint(
    websocket: WebSocket, 
    chat_id: int,
    token: str = Query(...)
):
    """
    Endpoint WebSocket para recibir streaming de respuestas del chat
    """
    connection_id = None
    
    try:
        # Validar token JWT
        try:
            payload = JWTHandler.decode_token(token)
            if not payload:
                await websocket.close(code=4001, reason="Token inválido")
                return
            
            # Verificar que el usuario existe en BD
            user = await chatbot_api.usuario.find(username=payload.username)
            if not user:
                await websocket.close(code=4001, reason="Usuario no encontrado")
                return
                
            user_id = payload.username
        except Exception as e:
            logger.error(f"Error validando token WebSocket: {e}")
            await websocket.close(code=4001, reason="Token inválido")
            return
        
        # Validar que el chat existe
        chat = await chatbot_api.chat.find(chat_id)
        if not chat:
            await websocket.close(code=4004, reason="Chat no encontrado")
            return
            
        # Validar permisos del chat
        if chat.username != user_id:
            await websocket.close(code=4003, reason="Sin permisos para este chat")
            return
        
        # Conectar WebSocket
        connection_id = await connection_manager.connect(websocket, user_id, chat_id)
        
        # Enviar confirmación de conexión
        await connection_manager.send_to_connection(
            connection_id,
            {
                "type": WebsocketMessageType.CONNECTION_ESTABLISHED.value,
                "meta": {
                    "chat_id": chat_id,
                    "connection_id": connection_id
                }
            }
        )
        
        # Mantener conexión viva
        while True:
            try:
                # Esperar mensajes del cliente (ej: ping, cancel, etc.)
                message = await websocket.receive_text()
                data: dict = json.loads(message)
                
                if data.get("type") == "ping":
                    await connection_manager.send_to_connection(
                        connection_id,
                        {"type": WebsocketMessageType.PONG.value, "meta": {"timestamp": datetime.now().isoformat()}}
                    )
                elif data.get("type") == "cancel_stream":
                    # TODO: Implementar cancelación de streaming
                    logger.info(f"Streaming cancelado por usuario en chat {chat_id}")
                    
            except WebSocketDisconnect:
                break
            except Exception as e:
                logger.error(f"Error en WebSocket para chat {chat_id}: {e}")
                break
    
    except Exception as e:
        logger.error(f"Error en conexión WebSocket: {e}")
        if websocket.client_state.name != "DISCONNECTED":
            await websocket.close(code=4000, reason="Error interno")
    
    finally:
        if connection_id:
            await connection_manager.disconnect(connection_id)


@streaming_router.delete("/chat/context/{chat_id}", response_model=APIResponse[bool])
async def clear_chat_context(
    chat_id: int,
    current_user: UsuarioRead = Depends(get_current_user)
) -> APIResponse[int]:
    """
    Limpiar contexto de un chat (elimina mensajes previos)
    
    Esto no elimina el chat ni los mensajes, solo el contexto guardado en la base de datos
    """
    # Validar que el chat existe y pertenece al usuario
    chat = await chatbot_api.chat.find(id=chat_id)
    if not chat:
        raise RecordNotFoundException(resource="Chat")
    
    if chat.username != current_user.username:
        raise APIException(message="No tienes permisos para este chat")
    
    # Limpiar contexto en la base de datos
    await llm_manager.delete_context(thread_id=str(chat.id))
    
    logger.info(f"Contexto del chat {chat.id} limpiado por usuario {current_user.username}")
    return APIResponse.success(
        data=True,
        message=f"Contexto del chat {chat.id} limpiado exitosamente"
    )

@streaming_router.get("/status", response_model=APIResponse[StreamingStatus])
async def get_streaming_status() -> APIResponse[StreamingStatus]:
    """Obtener estado del sistema de streaming"""
    status_data = StreamingStatus(
        status="active",
        message="Sistema de streaming operativo",
        connections=connection_manager.get_active_connections_count()
    )
    
    return APIResponse.success(
        data=status_data,
        message="Estado del sistema de streaming obtenido exitosamente"
    )

@streaming_router.get("/chat/{chat_id}/status", response_model=APIResponse[StreamingStatus])
async def get_chat_streaming_status(
    chat_id: int,
    current_user: UsuarioRead = Depends(get_current_user)
) -> APIResponse[StreamingStatus]:
    """Obtener estado de streaming para un chat específico"""
    
    # Validar permisos
    chat = await chatbot_api.chat.find(chat_id)
    if not chat:
        raise RecordNotFoundException(resource="Chat")
    
    if chat.username != current_user.username:
        raise APIException(message="No tienes permisos para este chat")
    
    status_data = StreamingStatus(
        status="active",
        message=f"Chat {chat_id} disponible para streaming",
        connections=connection_manager.get_chat_connections_count(chat_id),
        chat_id=chat_id
    )
    
    return APIResponse.success(
        data=status_data,
        message="Estado del chat obtenido exitosamente"
    )


from typing import Any, Optional, Dict, List, Union, Generic, TypeVar
from pydantic import BaseModel, Field, ConfigDict
from enum import Enum
from .exceptions import ErrorCode

T = TypeVar('T')

class ResponseStatus(str, Enum):
    """Estados posibles de una respuesta de la API."""
    SUCCESS = "success"
    ERROR = "error"
    WARNING = "warning"

class APIError(BaseModel):
    """Modelo para representar errores en la API."""
    code: ErrorCode = Field(..., description="Código de error específico")
    message: str = Field(..., description="Mensaje descriptivo del error")
    details: Optional[Dict[str, Any]] = Field(None, description="Detalles adicionales del error")
    field: Optional[str] = Field(None, description="Campo específico que causó el error")

class PaginationMeta(BaseModel):
    """Metadatos de paginación para respuestas que contienen listas de datos."""
    total: Optional[int] = Field(None, description="Total de registros disponibles")
    limit: Optional[int] = Field(None, description="Número máximo de elementos por página")
    offset: Optional[int] = Field(None, description="Número de elementos omitidos desde el inicio")
    has_next: Optional[bool] = Field(None, description="Indica si hay más páginas disponibles")
    has_prev: Optional[bool] = Field(None, description="Indica si hay páginas anteriores")

class APIResponse(BaseModel, Generic[T]):
    """
    Respuesta estandarizada de la API que envuelve todos los datos retornados.
    
    Esta clase proporciona un formato consistente para todas las respuestas de la API,
    incluyendo el estado de la operación, los datos solicitados, mensajes informativos
    y manejo de errores. El campo 'data' contiene el tipo específico solicitado (T)
    que puede ser cualquier modelo de Pydantic, permitiendo que la documentación
    refleje correctamente la estructura de los datos retornados.
    
    Attributes:
        status: Estado de la respuesta (success, error, warning)
        data: Datos de la respuesta del tipo especificado
        message: Mensaje descriptivo sobre el resultado de la operación
        errors: Lista de errores si la operación no fue exitosa
        meta: Metadatos adicionales como información de paginación
    """
    status: ResponseStatus = Field(
        ..., 
        description="Estado de la respuesta que indica el resultado de la operación"
    )
    data: Optional[T] = Field(
        None, 
        description="Datos de la respuesta del tipo especificado. La estructura exacta depende del endpoint consultado y se documenta en cada endpoint específico."
    )
    message: Optional[str] = Field(
        None, 
        description="Mensaje descriptivo sobre el resultado de la operación realizada"
    )
    errors: Optional[List[APIError]] = Field(
        None, 
        description="Lista de errores detallados si la operación no fue exitosa. Solo presente cuando status es 'error'."
    )
    meta: Optional[Dict[str, Any]] = Field(
        None, 
        description="Metadatos adicionales como información de paginación, contadores, etc."
    )

    model_config = ConfigDict(
        use_enum_values=True,
        validate_assignment=True,
    )

    @classmethod
    def success(
        cls, 
        data: T, 
        message: Optional[str] = None, 
        meta: Optional[Dict[str, Any]] = None
    ) -> 'APIResponse[T]':
        """Crea una respuesta exitosa."""
        return cls(
            status=ResponseStatus.SUCCESS,
            data=data,
            message=message,
            meta=meta
        )

    @classmethod
    def error(
        cls, 
        errors: Union[APIError, List[APIError]], 
        message: Optional[str] = None
    ) -> 'APIResponse[None]':
        """Crea una respuesta de error."""
        if isinstance(errors, APIError):
            errors = [errors]
        
        return cls(
            status=ResponseStatus.ERROR,
            data=None,
            message=message or "Se ha producido un error",
            errors=errors
        )

    @classmethod
    def not_found(cls, resource: str = "Registro") -> 'APIResponse[None]':
        """Crea una respuesta para recurso no encontrado."""
        return cls.error(
            APIError(
                code=ErrorCode.RECORD_NOT_FOUND,
                message=f"{resource} no encontrado"
            ),
            message=f"{resource} no encontrado"
        )

    @classmethod
    def validation_error(cls, field: str, message: str) -> 'APIResponse[None]':
        """Crea una respuesta de error de validación."""
        return cls.error(
            APIError(
                code=ErrorCode.VALIDATION_ERROR,
                message=message,
                field=field
            ),
            message="Error de validación"
        )

    @classmethod
    def database_error(cls, message: Optional[str] = None) -> 'APIResponse[None]':
        """Crea una respuesta de error de base de datos."""
        return cls.error(
            APIError(
                code=ErrorCode.DATABASE_ERROR,
                message=message or "Error en la base de datos"
            ),
            message="Error interno del servidor"
        )

class PaginatedResponse(APIResponse[List[T]]):
    """
    Respuesta paginada que extiende APIResponse para incluir metadatos de paginación.
    
    Esta clase se utiliza específicamente para endpoints que retornan listas de datos
    con soporte para paginación, incluyendo información sobre el total de registros,
    límites, navegación y metadatos útiles para implementar paginación en clientes.
    
    La información de paginación se incluye en el campo 'meta' bajo la clave 'pagination'
    y contiene detalles como el total de registros, límites aplicados y flags de navegación.
    
    Attributes:
        status: Siempre 'success' para respuestas paginadas exitosas
        data: Lista de elementos del tipo especificado
        message: Mensaje descriptivo de la operación
        meta: Contiene información de paginación en meta.pagination
    """
    
    @classmethod
    def success_paginated(
        cls,
        data: List[T],
        total: Optional[int] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        message: Optional[str] = None
    ) -> 'PaginatedResponse[T]':
        """Crea una respuesta exitosa paginada."""
        pagination_meta = PaginationMeta(
            total=total,
            limit=limit,
            offset=offset,
            has_next=None if total is None or limit is None or offset is None 
                     else (offset + limit) < total,
            has_prev=None if offset is None else offset > 0
        )
        
        return cls(
            status=ResponseStatus.SUCCESS,
            data=data,
            message=message,
            meta={"pagination": pagination_meta.model_dump()}
        )


from enum import Enum
from typing import Optional
from pydantic import BaseModel

class WebsocketMessageType(Enum):
    # LLM
    USER = "human"
    AI = "ai"
    AICHUNK = "AIMessageChunk"
    SYSTEM = "system"
    TOOL = "tool"
    TOOLCHUNK = "ToolMessageChunk"
    TOOLCALL = "tool_call"
    TOOLCALLCHUNK = "tool_call_chunk"
    # SERVICES
    CONNECTION_ESTABLISHED = "connection_established"
    PONG = "pong"
    MESSAGE_RECEIVED = "message_received"
    ASSISTANT_RESPONSE_START = "assistant_response_start"
    ASSISTANT_RESPONSE_COMPLETE = "assistant_response_complete"
    ASSISTANT_RESPONSE_ERROR = "assistant_response_error"

# Modelos de request/response
class StreamMessageRequest(BaseModel):
    content: str
    chat_id: int

class StreamMessageResponse(BaseModel):
    user_id: str
    msg_id: int
    connections_notified: int
    chat_id: int

class StreamingStatus(BaseModel):
    status: str
    message: str
    connections: int
    chat_id: Optional[int] = None


from .toolfuncs import *

#Añadir aquí las herramientas que quieras exponer
tools = [
    suma
] 

# Alternativamente, puedes importar todas las herramientas automáticamente
# Pero es menos eficiente y más propenso a errores

# # UNCOMMENT TO USE AUTOMATIC IMPORT OF TOOLS
# from . import toolfuncs
# from langchain_core.tools.structured import BaseTool

# tools = [
#     getattr(toolfuncs, name) 
#     for name in dir(toolfuncs) 
#     if isinstance(getattr(toolfuncs, name), BaseTool)
# ]



import sys
import click
import subprocess
from pathlib import Path
from tai_sql import pm
from tai_sql.cli.commands.cmd_deploy import push
from tai_sql.core.provider import Provider
from tai_sql.drivers import drivers
from sqlalchemy import create_engine, text

@click.command()
def pushdb():
    """Crea/empuja las bases de datos a un servidor remoto"""

    try:

        provider = Provider.from_environment('CONTEXT_DATABASE_URL')

        engine = create_engine(provider.url)

        driver = drivers.get_or_raise(provider.drivername)

        exists_query = driver.database_exists_query()


        with engine.connect() as conn:
            result = conn.execute(text(exists_query), {"db_name": provider.database})
            exists = result.fetchone() is not None
            if exists:
                click.echo(f"ℹ️  La base de datos '{provider.database}' ya existe")
            else:
                create_statement = driver.create_database_statement(provider.database)
                conn = conn.execution_options(isolation_level="AUTOCOMMIT")
                conn.execute(text(create_statement))
                click.echo(f"✅ Base de datos '{provider.database}' creada exitosamente")
            
            click.echo()

        database_dir = pm.find_project_root(Path(__file__).parent.parent.parent.parent)

        config = pm.load_config(database_dir)

        subprocess.run(['poetry', 'install'],
            cwd=database_dir,
            check=True, 
            capture_output=True)

        pm.set_current_schema(config.default_schema)

        push.callback(schema=None, force=True, dry_run=False, verbose=False, no_generate=False)

    except Exception as e:
        click.echo(f"❌ Error al inicializar el proyecto: {str(e)}", err=True)
        sys.exit(1)


from fastapi import APIRouter, Depends
from typing import List
from ..database import *
from ..resources import APIResponse

enumerations_router = APIRouter(
    prefix="/enums",
    tags=["Enumeraciones"]
)

@enumerations_router.get("/message-role", tags=["Enumeraciones"], response_model=APIResponse[List[str]])
async def get_message_role_enumeration(
    api: ChatbotAsyncDBAPI = Depends(ChatbotAsyncDBAPI)
) -> APIResponse[List[str]]:
    """
    Obtiene los valores de la enumeración message_role.
    """
    values = api.message_role.find_many()
    
    return APIResponse.success(
        data=values,
        message="Enumeración message_role obtenida exitosamente"
    )




"""
Custom generator for API routers.
"""
from tai_api.generators import RoutersGenerator


class CustomRoutersGenerator(RoutersGenerator):
    """
    TODO: Custom generator for API routers that extends the base RoutersGenerator.
    """
    pass

